from .auth import AuthMixin
from .bank_account_verify import AccountVerifyMixin
from .bank_statement import BankStatementMixin
from .bbps import BbpsMixin
from .cibil import CibilMixin
from .experian import ExperianMixin
from .gst_verificaion import GstVerifyMixin
from .udyog_aadhaar_verify import UdyogAadhaarVerifyMixin
from .highmark import HighmarkMixin
from .id_verification import IDVerificationMixin
from .mail import MailMixin
from .recharge import RechargeMixin
from .sms import SMSMixin
from .webhooks import WebhookMixin
from .insurance import InsuranceMixin
from .okyc_verification import OkycVerificationMixin
from .rc_verification import RcVerificationMixin
from .itr_pull import ItrVerificationMixin
from .itr_extraction import ItrExtractionMixin
from .equifax import EquifaxMixin
from .rekognition import RekognitionMixins
from .epfo_verification import EpfoVerificationMixin
from .name_matching import NameMatchMixin
from .e_sign_verification import ESignVerification
from .ckyc import CkycMixin

__all__ = [
    AuthMixin,
    BankStatementMixin,
    CibilMixin,
    ExperianMixin,
    HighmarkMixin,
    IDVerificationMixin,
    MailMixin,
    SMSMixin,
    WebhookMixin,
    InsuranceMixin,
    RechargeMixin,
    BbpsMixin,
    AccountVerifyMixin,
    GstVerifyMixin,
    UdyogAadhaarVerifyMixin,
    OkycVerificationMixin,
    RcVerificationMixin,
    EquifaxMixin,
    RekognitionMixins,
    ItrVerificationMixin,
    ItrExtractionMixin,
    EpfoVerificationMixin,
    NameMatchMixin,
    ESignVerification,
    CkycMixin,
]



import re
import hmac
import hashlib


class WebhookMixin:
    """
    Mixin for working with webhooks
    """

    HEADER_REGEX = r"^t=(\d+),s=(\w+)$"

    def verify_webhook(self, request_body, signature_header, webhook_secret):
        """
        Verify webhook request

        :param request_body: Raw request body of webhook as bytes
        :type request_body: bytes
        :param signature_header: Signature header of request. Mercury signature header
            is Mercury-Signature
        :type signature_header: str
        :param webhook_secret: Secret configured in Mercury for the user
        :type webhook_secret: str
        :return: Boolean of whether the webhook request is valid
        :rtype: bool
        """

        timestamp, signature = self._extract_timestamp_signature(signature_header)

        digest_message = "{}.{}".format(timestamp, request_body.decode("utf-8"))
        generated_signature = self._generate_signature(webhook_secret, digest_message)
        return hmac.compare_digest(signature, generated_signature)

    def _generate_signature(self, webhook_secret, digest_message):
        """
        generate_signature Generate signature from secret and message

        :param webhook_secret: Key for the digest
        :type webhook_secret: str
        :param digest_message: Message fto be digested
        :type digest_message: str
        :return: Generated signature
        :rtype: str
        """
        return hmac.new(
            webhook_secret.encode("utf-8"),
            digest_message.encode("utf-8"),
            hashlib.sha256,
        ).hexdigest()

    def _extract_timestamp_signature(self, signature_header):
        """
        Get timestamp and signature from Mercury-Signature header

        :param signature_header: Value of Mercury-Signature header form webhook
        :type signature_header: str
        :return: Tuple containing timestamp and signature
        :rtype: (str, str)
        """
        m = re.match(self.HEADER_REGEX, signature_header)
        if m is None:
            raise ValueError("Signature header is invalid")

        timestamp = m.group(1)
        signature = m.group(2)
        return timestamp, signature



from unittest import mock
from unittest import TestCase

from mercuryclient.api import MercuryApi
from mercuryclient.types.highmark.request import (
    Address,
    Applicant,
    HighmarkRequest,
    Identity,
    Relation,
)


class HighmarkMixinTest(TestCase):
    def setUp(self):
        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        self.request_obj = HighmarkRequest(
            inquiry_reference_number="ref_no",
            credit_request_type="INDIVIDUAL",
            credit_transaction_id="transaction_id",
            inquiry_purpose_type="ACCT-ORIG",
            inquiry_purpose_type_desc="AUTO_LOAN",
            inquiry_stage="PRE_SCREEN",
            los_app_id="RANDOM_APP_ID",
            loan_amount=100,
            applicant=Applicant(
                name="Sample Name",
                date_of_birth="2000-01-01",
                nominee=Relation(relation_type="FATHER", relation_name="Mr. Father"),
                identities=[Identity(id_type="PAN_CARD", id_number="ABCDE123")],
                addresses=[
                    Address(
                        address_type="RESIDENCE",
                        address_line_1="Random Address",
                        city="Random City",
                        state="BIHAR",
                        pincode="123456",
                    )
                ],
            ),
        )
        self.request_dict = {
            "profile": "some_profile",
            "inquiry_reference_number": "ref_no",
            "credit_request_type": "INDIVIDUAL",
            "credit_transaction_id": "transaction_id",
            "inquiry_purpose_type": "ACCT-ORIG",
            "inquiry_purpose_type_desc": "AUTO_LOAN",
            "inquiry_stage": "PRE_SCREEN",
            "los_app_id": "RANDOM_APP_ID",
            "loan_amount": 100,
            "applicant": {
                "name": "Sample Name",
                "date_of_birth": "2000-01-01",
                "identities": [{"id_type": "PAN_CARD", "id_number": "ABCDE123"}],
                "nominee": {"relation_type": "FATHER", "relation_name": "Mr. Father"},
                "addresses": [
                    {
                        "address_type": "RESIDENCE",
                        "address_line_1": "Random Address",
                        "city": "Random City",
                        "state": "BIHAR",
                        "pincode": "123456",
                    }
                ],
            },
        }

        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)
        self.get_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._get_json_http_request"
        ).start()
        self.addCleanup(self.get_api_mock.stop)
        self.sleep_mock = mock.patch("mercuryclient.mixins.highmark.time.sleep").start()
        self.sleep_mock.return_value = None
        self.addCleanup(self.get_api_mock.stop)

    def test_request_highmark_report(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        self.client.request_highmark_report(self.request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            "api/v1/highmark/",
            data=self.request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)

        with self.assertRaises(Exception):
            self.client.request_highmark_report(self.request_obj, "some_profile")

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        request_id = self.client.request_highmark_report(
            self.request_obj, "some_profile"
        )
        self.assertEqual(request_id, "random_string")

    def test_response_highmark_report(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        self.get_api_mock.return_value = ("random_string", mock_response)
        self.client.get_highmark_response("random_string")

        self.get_api_mock.assert_called_with(
            "api/v1/highmark/",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )

    def test_response_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.get_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.get_highmark_response("random_string")

    def test_response_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        mock_response.json = mock.MagicMock()
        mock_response.json.return_value = {"status": "SUCCESS"}
        self.get_api_mock.return_value = ("random_string", mock_response)

        request_id, result = self.client.get_highmark_response("random_string")
        self.assertEqual(request_id, "random_string")
        self.assertEqual(result["status"], "SUCCESS")

    def test_entire_request_response_flow(self):
        mock_post_response = mock.MagicMock()
        mock_post_response.status_code = 201
        mock_get_response = mock.MagicMock()
        mock_get_response.status_code = 200
        mock_get_response.json = mock.MagicMock()
        mock_get_response.json.return_value = {"status": "SUCCESS"}
        self.post_api_mock.return_value = ("random_string", mock_post_response)
        self.get_api_mock.return_value = ("random_string", mock_get_response)

        response = self.client.fetch_highmark_report(self.request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            "api/v1/highmark/",
            data=self.request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )
        self.get_api_mock.assert_called_with(
            "api/v1/highmark/",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )
        self.sleep_mock.assert_called_with(11)
        self.assertEqual(response["status"], "SUCCESS")

    def test_entire_request_response_flow_failure(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        mock_post_response = mock.MagicMock()
        mock_post_response.status_code = 201
        mock_get_response = mock.MagicMock()
        mock_get_response.status_code = 200
        mock_get_response.json = mock.MagicMock()
        mock_get_response.json.return_value = {"status": "IN_PROGRESS"}
        self.post_api_mock.return_value = ("random_string", mock_post_response)
        self.get_api_mock.return_value = ("random_string", mock_get_response)

        with self.assertRaises(Exception):
            client.fetch_highmark_report(self.request_obj, "some_profile")



import time
from typing import Tuple


class BankStatementMixin:
    """
    Mixin for analyzing bank statements
    """

    def request_bank_statement(self, payload) -> str:
        api = "api/v1/bank_statement/"

        request_id, r = self._post_json_http_request(
            api, data=payload, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:
            return request_id

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending bank statement verififcation request "
            "Status: {}, Response is {}".format(r.status_code, response_json)
        )

    def get_bank_statement_result(self, request_id: str) -> Tuple[str, dict]:
        api = "api/v1/bank_statement/"

        request_id, r = self._get_json_http_request(
            api,
            headers={"X-Mercury-Request-Id": request_id},
            send_request_id=False,
            add_bearer_token=True,
        )
        if r.status_code == 200:
            result = r.json()
            return request_id, result

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error getting bank statement Verification result. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def analysis_bank_statement(
        self,
        provider: str,
        profile: str,
        statement_file_url: str,
        statement_bank_name: str = None,
        statement_password: str = None,
        entity_type: str = None,
        bank_code: str = None,
        account_type: str = None,
        account_number: str = None,
        max_attempts: int = 5,
        retry_backoff: int = 5,
    ):

        request_data = {
            "provider": provider,
            "profile": profile,
            "statement_file_url": statement_file_url,
        }
        if statement_password is not None:
            request_data["statement_password"] = statement_password
        if statement_bank_name is not None:
            request_data["statement_bank_name"] = statement_bank_name

        if provider == "signzy":
            if not all([entity_type, bank_code, account_type, account_number]):
                raise ValueError(
                    "Mandatory fields missing: entity_type,bank_code,account_type,account_number required for signzy"
                )
            request_data.update(
                {
                    "entity_type": entity_type,
                    "bank_code": bank_code,
                    "account_type": account_type,
                    "account_number": account_number,
                }
            )

        print(request_data)
        mercury_request_id = self.request_bank_statement(request_data)

        for attempts in range(max_attempts):
            time.sleep(retry_backoff)
            mercury_request_id, result = self.get_bank_statement_result(
                mercury_request_id
            )
            if result.get("status") != "IN_PROGRESS":
                return mercury_request_id, result

            retry_backoff *= 2

        raise Exception(
            "Error while getting bank statement verification. Status: IN_PROGRESS"
        )



import logging

logger = logging.getLogger(__name__)


class RechargeMixin:
    def get_operators_list(self, profile: str):

        """
        Get oparators list using the mercury service
        :param profile: An existing profile in Mercury.
        :return: (request_id, status,result )
        """
        api = "api/v1/mobile_recharge/get_providers/"
        request_dict = {}
        request_dict["profile"] = profile
        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )
        if r.status_code == 201:
            return {"request_id": request_id, "data": r.content}
        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending get provider Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def make_recharge(self, rechagre_data, profile):

        """
        Make recharge using mercury service
        :param rechagre_data: data for recharge
        :param Profile: An existing profile in Mercury.
        """
        api = "api/v1/mobile_recharge/recharge/"
        rechagre_data["profile"] = profile
        request_id, r = self._post_json_http_request(
            path=api, data=rechagre_data, send_request_id=True, add_bearer_token=True
        )
        if r.status_code == 201:
            return {"request_id": request_id, "data": r.content}
        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending get provider Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_recharge_status(self, status_data, profile):

        """
        Make recharge using mercury service
        :param status_data: data for recharge
        :param Profile: An existing profile in Mercury.
        """
        api = "api/v1/mobile_recharge/status/"
        status_data["profile"] = profile
        request_id, r = self._post_json_http_request(
            path=api, data=status_data, send_request_id=True, add_bearer_token=True
        )
        if r.status_code == 201:
            return {"request_id": request_id, "data": r.content}
        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending get provider Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_recharge_wallet_balance(self, profile):

        """
        check wallet balance using mercury service
        :param Profile: An existing profile in Mercury.
        """
        api = "api/v1/mobile_recharge/balance/"
        request_dict = {}
        request_dict["profile"] = profile
        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )
        if r.status_code == 201:
            return {"request_id": request_id, "data": r.content}
        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending get rechagre balance Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_recharge_ip(self, profile):

        """
        get ip of securepayment using mercury service
        :param Profile: An existing profile in Mercury.
        """
        api = "api/v1/mobile_recharge/ip/"
        request_dict = {}
        request_dict["profile"] = profile
        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )
        print(f"Error {request_dict} {r} {r.status_code}")
        if r.status_code == 201:
            return {"request_id": request_id, "data": r.content}
        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending fetching IP Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_recharge_plans(self, operator, profile):
        """
        Make recharge using mercury service
        :param operator: data for recharge
        :param Profile: An existing profile in Mercury.
        """
        api = "api/v1/mobile_recharge/plans/"
        operator["profile"] = profile
        request_id, r = self._post_json_http_request(
            path=api, data=operator, send_request_id=True, add_bearer_token=True
        )
        if r.status_code == 201:
            return {"request_id": request_id, "data": r.content}
        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending get provider Request Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )



from unittest import mock
from unittest import TestCase

from mercuryclient.api import MercuryApi


class SMSMixinTest(TestCase):
    def setUp(self):
        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)
        self.get_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._get_json_http_request"
        ).start()
        self.addCleanup(self.get_api_mock.stop)

    def test_send_sms_request(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        client.send_sms("9876543210", "Random message", "some_provider", "some_profile")

        self.post_api_mock.assert_called_with(
            "api/v1/sms/",
            data={
                "provider": "some_provider",
                "profile": "some_profile",
                "recipient": "9876543210",
                "message": "Random message",
            },
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            client.send_sms(
                "9876543210", "Random message", "some_provider", "some_profile"
            )

    def test_request_api_succeeds_if_status_code_success(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        response = client.send_sms(
            "9876543210", "Random message", "some_provider", "some_profile"
        )
        self.assertEqual(response["request_id"], "random_string")
        self.assertEqual(response["status"], "Success")

    def test_response_sms_result(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        self.get_api_mock.return_value = ("random_string", mock_response)
        client.get_sms_result("random_string")

        self.get_api_mock.assert_called_with(
            "api/v1/sms/",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )

    def test_response_exception_raised_if_status_code_error(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.get_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            client.get_sms_result("random_string")

    def test_response_api_succeeds_if_status_code_success(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        mock_response.json = mock.MagicMock()
        mock_response.json.return_value = {"status": "SUCCESS"}
        self.get_api_mock.return_value = ("random_string", mock_response)

        response = client.get_sms_result("random_string")
        self.assertEqual(response["request_id"], "random_string")
        self.assertEqual(response["status"], "SUCCESS")
        self.assertEqual(response["data"], {"status": "SUCCESS"})



from unittest import mock
from unittest import TestCase

from mercuryclient.api import MercuryApi


class NameMatchingMixinTest(TestCase):
    def setUp(self):

        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )

        self.name_1 = "MRs. test name"
        self.name_2 = "Mr test name"

        self.expected_dict = {
            "profile": "some_profile",
            "provider": "some_provider",
            "primary_name": self.name_1,
            "secondary_name": self.name_2,
            "remove_salutations": True,
        }

        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)

    def test_request_name_match(self):

        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        self.post_api_mock.return_value = ("random_string", mock_response)
        self.client.name_match(
            "some_provider",
            "some_profile",
            primary_name=self.name_1,
            secondary_name=self.name_2,
        )
        self.post_api_mock.assert_called_with(
            path="api/v1/name/match",
            data=self.expected_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.name_match(
                "some_provider",
                "some_profile",
                primary_name=self.name_1,
                secondary_name=self.name_2,
            )

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_id, response = self.client.name_match(
            "some_provider",
            "some_profile",
            primary_name=self.name_1,
            secondary_name=self.name_2,
        )
        self.assertEqual(request_id, "random_string")



from dataclasses import field
from typing import List, Optional
from pydantic import BaseModel
from pydantic import Field


class Premium(BaseModel):

    amount: int


class Customer(BaseModel):

    id: Optional[str] = Field(max_length=50)
    name: str = Field(max_length=50)
    phone: str = Field(max_length=50)
    email: Optional[str] = Field(max_length=50)
    state: str = Field(max_length=50)


class AccountData(BaseModel):

    number: str = Field(max_length=50)
    ifsc: str = Field(max_length=50)
    category: Optional[str] = Field(max_length=50)


class Insured(BaseModel):

    name: str = Field(max_length=50)
    phone: str = Field(max_length=50)
    email: str = Field(max_length=50)
    pincode: Optional[str] = Field(max_length=50)
    city: Optional[str] = Field(max_length=50)
    address: Optional[str] = Field(max_length=50)
    gender: Optional[str] = Field(max_length=50)
    occupation: Optional[str] = Field(max_length=50)
    age: int = Field(min=18, max=50)
    dob: str = Field(max_length=50)
    account: AccountData


class Nominee(BaseModel):

    name: str = Field(max_length=50)
    relationship: str = Field(max_length=50)
    phone: str = Field(max_length=50)
    dob: Optional[str] = Field(max_length=50)


class PersonData(BaseModel):
    insured: Insured
    nominee: Nominee


class LoanData(BaseModel):

    amount: int
    provider_id: int
    provider_name: str = Field(max_length=50)
    start_date: str = Field(max_length=50)
    end_date: str = Field(max_length=50)
    emi_amount: Optional[int]
    hospicash_amount: Optional[int]
    disbursement_date: str = Field(max_length=50)
    account_number: str = Field(max_length=50)
    tenure_in_months: int
    type: Optional[str] = Field(max_length=50)
    family_type: str = Field(max_length=50)
    application_number: str = Field(max_length=50)
    person: List[PersonData]


class AdditionalInfo(BaseModel):

    declaration_health: str = Field(max_length=4)
    assignment_clause: str = Field(max_length=4)
    collection_status: str = Field(max_length=4)


class InsuranceRequest(BaseModel):

    plan: str = Field(max_length=50)
    partner_id: str = Field(max_length=50)
    plan_type: int
    reference_id: str = Field(max_length=50)
    category: str = Field(max_length=50)
    premium: Premium
    customer: Customer
    loan: LoanData
    additional_info: AdditionalInfo



class GstVerifyMixin:
    """
    Mixin for gst verification
    """

    def verify_gstin(self, gstin_number: str, provider: str, profile: str):
        """
        gstin verification using the mercury service

        :param request_obj: GstVerifyRequest
        :param provider: any
        :param profile: An existing profile in Mercury. The profile has to correspond
        to the provider.
        :return: (request_id, data)
        """
        api = "api/v1/gst_verification/"

        request_dict = {
            "profile": profile,
            "provider": provider,
            "gstin_number": gstin_number,
        }

        request_id, r = self._post_json_http_request(
            path=api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        try:
            response_json = r.json()
        except Exception:
            response_json = {}

        if r.status_code == 201:
            return {"request_id": request_id, "data": response_json}

        raise Exception(
            "Error while sending gstin verify request. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_verify_gst_result(self, request_id: str):
        api = "api/v1/gst_verification/"

        request_id, r = self._get_json_http_request(
            api,
            headers={"X-Mercury-Request-Id": request_id},
            send_request_id=False,
            add_bearer_token=True,
        )

        if r.status_code == 200:
            result = r.json()
            if result["status"] == "FAILURE":
                raise Exception(
                    "Error verifying Gst. Status: {} | Message {}".format(
                        result["status"], result["message"]
                    )
                )
            return request_id, result

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error getting Gst Verification result. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )



from unittest import TestCase

from mercuryclient.api import MercuryApi


class WebhookMixinTest(TestCase):
    def test_extract_signature_timestamp(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        sample_header = "t=677548800,s=3c0098c1f95739fc8167fb1bb05d014066d8381fc6a50a03ab665a57e239042a"
        ts, sign = client._extract_timestamp_signature(sample_header)

        self.assertEqual(ts, "677548800")
        self.assertEqual(
            sign, "3c0098c1f95739fc8167fb1bb05d014066d8381fc6a50a03ab665a57e239042a"
        )

    def test_verify_webhook(self):
        client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )

        sample_header = "t=677548800,s=3c0098c1f95739fc8167fb1bb05d014066d8381fc6a50a03ab665a57e239042a"
        sample_body = b'{"sample_key": "sample_value"}'
        sample_secret = "SAMPLE_KEY"

        self.assertTrue(
            client.verify_webhook(sample_body, sample_header, sample_secret)
        )

        sample_secret = "INCORRECT_KEY"
        self.assertFalse(
            client.verify_webhook(sample_body, sample_header, sample_secret)
        )



import time
import json
from typing import Tuple

from mercuryclient.types.equifax.request import EquifaxRequest


class EquifaxMixin:
    """
    Mixin for registering Equifax requests
    """

    def request_equifax_report(self, request_obj: EquifaxRequest, profile: str) -> str:
        """
        POST a request to Mercury to Request Equifax report. This posts the request to
        Mercury and returns immediately. Use the returned request ID and poll
        get_equifax_response to check for report.
        You can also use fetch_equifax_report which is a helper function that combines
        this api and the result api to get the result.

        :param request_obj: Object of EquifaxRequest model
        :type request_obj: EquifaxRequest
        :param profile: Equifax profile name
        :type profile: str
        :return: Request ID
        :rtype: str
        """
        api = "api/v1/equifax/"

        # It is necessary to JSON encode the models to serialize the enums and datetime
        # formats into strings
        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile
        request_id, r = self._post_json_http_request(
            api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:
            return request_id

        try:
            response_json = r.json()
        except Exception:
            response_json = r.text
        raise Exception(
            "Error while sending Equifax request. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_equifax_response(self, request_id: str) -> Tuple[str, dict]:
        """
        Get result for Equifax job request at the provided request ID. You can poll
        this method to check for response. If the job is still progressing, the status
        within the response will be IN_PROGRESS and the method can continue to be polled
        until you get either a SUCCESS or FAILURE

        :param request_id: Request ID of the Equifax job request
        :type request_id: str
        :return: Tuple containing request ID and job response. This response will
            contain the "status" of the job (IN_PROGRESS, SUCCESS or FAILURE), a
            "message" and a "data" key containing a dict with the bureau report
        :rtype: Tuple[str, dict]
        """
        api = "api/v1/equifax/"

        request_id, r = self._get_json_http_request(
            api,
            headers={"X-Mercury-Request-Id": request_id},
            send_request_id=False,
            add_bearer_token=True,
        )

        if r.status_code == 200:
            return request_id, r.json()

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while getting Equifax response. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def fetch_equifax_report(
        self,
        request_obj: EquifaxRequest,
        profile: str,
        max_attempts: int = 8,
        retry_backoff: int = 15,
    ) -> dict:
        """
        Generate an Equifax request and get job result

        :param request_obj: Object of EquifaxRequest model
        :type request_obj: EquifaxRequest
        :param profile: Equifax profile name
        :type profile: str
        :param max_attempts: Number of attempts to make when fetching the result,
            defaults to 8
        :type max_attempts: int, optional
        :param retry_backoff: Number of seconds to backoff when retrying to get the
            result, defaults to 15
        :type retry_backoff: int, optional
        :return: Dict containing the job result
        :rtype: dict
        """

        request_id = self.request_equifax_report(request_obj, profile)
        attempts = 0
        while attempts < max_attempts:
            time.sleep(retry_backoff)
            request_id, result = self.get_equifax_response(request_id)
            if result.get("status") != "IN_PROGRESS":
                return result

            retry_backoff *= 2
            attempts += 1

        raise Exception("Error while getting Equifax response. Status: IN_PROGRESS")



import time
from typing import Tuple

from mercuryclient.types.epfo_verification.enums import RequestTypes


class EpfoVerificationMixin:
    """
    Mixin for verifying Epfo
    """

    def _initiate_epfo_request(
        self,
        request_data,
    ) -> str:
        api = "api/v1/epfo/verify/"
        try:
            RequestTypes(request_data.get("request_type"))
        except ValueError:
            raise Exception(
                f"{request_data.get('request_type')} is not a valid request_type"
            )

        request_id, r = self._post_json_http_request(
            api, data=request_data, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:
            return request_id

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending Epfo verification request. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def _get_epfo_response(self, request_id: str) -> Tuple[str, dict]:
        api = "api/v1/epfo/verify/"

        request_id, r = self._get_json_http_request(
            api,
            headers={"X-Mercury-Request-Id": request_id},
            send_request_id=False,
            add_bearer_token=True,
        )

        if r.status_code == 200:
            result = r.json()
            return request_id, result

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error getting Epfo Verification result. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def _get_epfo_result(
        self, mercury_request_id: str, max_attempts: int, retry_backoff: int
    ):
        """Retry mechanism for epfo response

        raises:
            Exception: After max attempts, response till remains in progress

        returns mercury_request_id,response from the vendor

        """

        for _ in range(max_attempts):
            time.sleep(retry_backoff)
            mercury_request_id, result = self._get_epfo_response(mercury_request_id)
            if result.get("status") != "IN_PROGRESS":
                return mercury_request_id, result

            retry_backoff *= 2

        raise Exception("Error while getting Epfo response. Status: IN_PROGRESS")

    def generate_epfo_otp(
        self,
        profile: str,
        provider: str,
        mobile_number: str,
        callback_url: str = None,
        max_attempts: int = 3,
        retry_backoff: int = 5,
    ):
        """Initiates the request for epfo otp generation.

        returns: mercury_request_id,otp generation response from vendor
        """
        request_data = {
            "provider": provider,
            "profile": profile,
            "mobile_number": mobile_number,
            "request_type": RequestTypes.OTP_REQUEST.value,
        }
        if callback_url:
            request_data["callback_url"] = callback_url
        mercury_request_id = self._initiate_epfo_request(request_data)
        return self._get_epfo_result(mercury_request_id, max_attempts, retry_backoff)

    def verify_epfo_otp(
        self,
        profile: str,
        provider: str,
        otp: str,
        epfo_request_id: str,
        max_attempts: int = 3,
        retry_backoff: int = 5,
    ):
        """Get OTP from customer and verify with request_id.

        returns: mercury_request_id, verify epfo response from vendor
        """
        request_data = {
            "provider": provider,
            "profile": profile,
            "otp": otp,
            "request_id": epfo_request_id,
            "request_type": RequestTypes.OTP_VERIFY.value,
        }
        mercury_request_id = self._initiate_epfo_request(request_data)
        return self._get_epfo_result(mercury_request_id, max_attempts, retry_backoff)

    def get_epfo_details(
        self,
        profile: str,
        provider: str,
        epfo_request_id: str,
        max_attempts: int = 3,
        retry_backoff: int = 5,
    ):
        """Get OTP from customer and verify with request_id.

        returns: mercury_request_id, verify epfo response from vendor
        """
        request_data = {
            "provider": provider,
            "profile": profile,
            "request_id": epfo_request_id,
            "request_type": RequestTypes.EPFO_DETAILS.value,
        }
        mercury_request_id = self._initiate_epfo_request(request_data)
        return self._get_epfo_result(mercury_request_id, max_attempts, retry_backoff)



from datetime import date
from typing import List
from typing import Optional

from pydantic import BaseModel
from pydantic import Field

from .enums import (
    AdditionalNameType,
    IDTypes,
    ProductCode,
    InquiryPurpose,
    State,
    AddressType,
    PhoneNumberType,
)

from ..common import Pincode


class Address(BaseModel):
    address_line_1: str = Field(max_length=200)
    address_type: List[AddressType]
    locality: Optional[str] = Field(max_length=40)
    city: Optional[str] = Field(max_length=50)
    state: State
    postal: Pincode


class PhoneNumber(BaseModel):

    number: str = Field(min_length=5, max_length=15)
    number_type: List[PhoneNumberType]


class Identity(BaseModel):
    id_value: str = Field(max_length=20)
    id_type: IDTypes
    source: Optional[str]


class FamilyDetails(BaseModel):
    additional_name_type: AdditionalNameType
    additional_name: str = Field(max_length=200)


class MIFDetails(BaseModel):
    family_details: List[FamilyDetails]


class EquifaxRequest(BaseModel):
    product_code: List[ProductCode] = Field(min_items=1, max_items=3)
    inquiry_purpose: InquiryPurpose
    first_name: str = Field(max_length=40)
    middle_name: Optional[str] = Field(max_length=40)
    last_name: str = Field(max_length=40)
    dob: date
    inquiry_address: List[Address] = Field(min_items=1, max_items=3)
    inquiry_phones: List[PhoneNumber] = Field(min_items=1, max_items=3)
    id_details: List[Identity] = Field(min_items=1, max_items=10)
    mif_details: MIFDetails



from enum import Enum


class Purpose(Enum):
    AUTO_LOAN = "AUTO_LOAN"
    HOUSING_LOAN = "HOUSING_LOAN"
    PROPERTY_LOAN = "PROPERTY_LOAN"
    LOAN_AGAINST_SECURITIES = "LOAN_AGAINST_SECURITIES"
    PERSONAL_LOAN = "PERSONAL_LOAN"
    CONSUMER_LOAN = "CONSUMER_LOAN"
    GOLD_LOAN = "GOLD_LOAN"
    EDUCATION_LOAN = "EDUCATION_LOAN"
    LOAN_TO_PROFESSIONAL = "LOAN_TO_PROFESSIONAL"
    CREDIT_CARD = "CREDIT_CARD"
    LEASING = "LEASING"
    OVERDRAFT = "OVERDRAFT"
    TWO_WHEELER_LOAN = "TWO_WHEELER_LOAN"
    NON_FUNDED_CREDIT_FACILITY = "NON_FUNDED_CREDIT_FACILITY"
    LOAN_AGAINST_BANK_DEPOSITS = "LOAN_AGAINST_BANK_DEPOSITS"
    FLEET_CARD = "FLEET_CARD"
    COMMERCIAL_VEHICLE_LOAN = "COMMERCIAL_VEHICLE_LOAN"
    TELCO_WIRELESS = "TELCO_WIRELESS"
    TELCO_BROADBAND = "TELCO_BROADBAND"
    TELCO_LANDLINE = "TELCO_LANDLINE"
    SECURED_CREDIT_CARD = "SECURED_CREDIT_CARD"
    USED_CAR_LOAN = "USED_CAR_LOAN"
    CONSTRUCTION_EQUIPMENT_LOAN = "CONSTRUCTION_EQUIPMENT_LOAN"
    TRACTOR_LOAN = "TRACTOR_LOAN"
    CORPORATE_CREDIT_CARD = "CORPORATE_CREDIT_CARD"
    KISAN_CREDIT_CARD = "KISAN_CREDIT_CARD"
    LOAN_ON_CREDIT_CARD = "LOAN_ON_CREDIT_CARD"
    PRIME_MINISTER_JAAN_DHAN_YOJANA_OVERDRAFT = (
        "PRIME_MINISTER_JAAN_DHAN_YOJANA_OVERDRAFT"
    )
    MUDRA_LOANS = "MUDRA_LOANS"
    MICROFINANCE_BUSINESS_LOAN = "MICROFINANCE_BUSINESS_LOAN"
    MICROFINANCE_PERSONAL_LOAN = "MICROFINANCE_PERSONAL_LOAN"
    MICROFINANCE_HOUSING_LOAN = "MICROFINANCE_HOUSING_LOAN"
    MICROFINANCE_OTHER = "MICROFINANCE_OTHER"
    PRADHAN_MANTRI_AWAS_YOJANA_MAY_CLSS = "PRADHAN_MANTRI_AWAS_YOJANA_MAY_CLSS"
    BUSINESS_LOAN_SECURED = "BUSINESS_LOAN_SECURED"
    BUSINESS_LOAN_GENERAL = "BUSINESS_LOAN_GENERAL"
    BUSINESS_LOAN_PRIORITY_SMALL_BUSINESS = "BUSINESS_LOAN_PRIORITY_SMALL_BUSINESS"
    BUSINESS_LOAN_PRIORITY_AGRICULTURE = "BUSINESS_LOAN_PRIORITY_AGRICULTURE"
    BUSINESS_LOAN_PRIORITY_OTHERS = "BUSINESS_LOAN_PRIORITY_OTHERS"
    BUSINESS_NON_FUNDED_CREDIT_GENERAL = "BUSINESS_NON_FUNDED_CREDIT_GENERAL"
    BUSINESS_NON_FUNDED_CREDIT_PRIORITY_SMALL = (
        "BUSINESS_NON_FUNDED_CREDIT_PRIORITY_SMALL"
    )
    BUSINESS_NON_FUNDED_CREDIT_PRIORITY_AGRICULTURE = (
        "BUSINESS_NON_FUNDED_CREDIT_PRIORITY_AGRICULTURE"
    )
    BUSINESS_NON_FUNDED_CREDIT_PRIORITY_OTHERS = (
        "BUSINESS_NON_FUNDED_CREDIT_PRIORITY_OTHERS"
    )
    BUSINESS_LOAN_AGAINST_BANK_DEPOSITS = "BUSINESS_LOAN_AGAINST_BANK_DEPOSITS"
    BUSINESS_LOAN_UNSECURED = "BUSINESS_LOAN_UNSECURED"
    MICROFINANCE_DETAILED_REPORT = "MICROFINANCE_DETAILED_REPORT"
    SUMMARY_REPORT = "SUMMARY_REPORT"
    LOCATE_PLUS_FOR_INSURANCE = "LOCATE_PLUS_FOR_INSURANCE"
    ACCOUNT_REVIEW = "ACCOUNT_REVIEW"
    RETRO_ENQUIRY = "RETRO_ENQUIRY"
    LOCATE_PLUS = "LOCATE_PLUS"
    ADVISER_LIABILITY = "ADVISER_LIABILITY"
    SECURED = "SECURED"
    UNSECURED = "UNSECURED"
    OTHER = "OTHER"


class State(Enum):
    ANDHRA_PRADESH = "ANDHRA_PRADESH"
    ARUNACHAL_PRADESH = "ARUNACHAL_PRADESH"
    ASSAM = "ASSAM"
    BIHAR = "BIHAR"
    CHATTISGARH = "CHATTISGARH"
    GOA = "GOA"
    GUJARAT = "GUJARAT"
    HARYANA = "HARYANA"
    HIMACHAL_PRADESH = "HIMACHAL_PRADESH"
    JAMMU_KASHMIR = "JAMMU_KASHMIR"
    JHARKHAND = "JHARKHAND"
    KARNATAKA = "KARNATAKA"
    KERALA = "KERALA"
    MADHYA_PRADESH = "MADHYA_PRADESH"
    MAHARASHTRA = "MAHARASHTRA"
    MANIPUR = "MANIPUR"
    MEGHALAYA = "MEGHALAYA"
    MIZORAM = "MIZORAM"
    NAGALAND = "NAGALAND"
    ORISSA = "ORISSA"
    PUNJAB = "PUNJAB"
    RAJASTHAN = "RAJASTHAN"
    SIKKIM = "SIKKIM"
    TAMIL_NADU = "TAMIL_NADU"
    TELANGANA = "TELANGANA"
    TRIPURA = "TRIPURA"
    UTTAR_PRADESH = "UTTAR_PRADESH"
    UTTARANCHAL = "UTTARANCHAL"
    WEST_BENGAL = "WEST_BENGAL"
    ANDAMAN_NICOBAR_ISLANDS = "ANDAMAN_NICOBAR_ISLANDS"
    CHANDIGARH = "CHANDIGARH"
    DADRA_NAGAR_HAVELI = "DADRA_NAGAR_HAVELI"
    DAMAN_DIU = "DAMAN_DIU"
    DELHI = "DELHI"
    LAKSHADWEEP = "LAKSHADWEEP"
    PONDICHERRY = "PONDICHERRY"
    APO = "APO"


class ScoreType(Enum):
    NO_SCORE = "NO_SCORE"
    CIBILTUSCR_ONLY = "CIBILTUSCR_ONLY"
    PLSCORE_ONLY = "PLSCORE_ONLY"
    CIBILTUSCR_PLSCORE = "CIBILTUSCR_PLSCORE"
    CIBILTUSC2_ONLY = "CIBILTUSC2_ONLY"
    CIBILTUSC2_PLSCORE = "CIBILTUSC2_PLSCORE"
    CIBILTUSC3_ONLY = "CIBILTUSC3_ONLY"
    CIBILTUSC3_PLSCORE = "CIBILTUSC3_PLSCORE"
    CIBILTUIE1_CIBILTUSCR = "CIBILTUIE1_CIBILTUSCR"
    CIBILTUIE1_CIBILTUSC2 = "CIBILTUIE1_CIBILTUSC2"
    CIBILTUIE1_CIBILTUSC3 = "CIBILTUIE1_CIBILTUSC3"
    CIBILTUIE1_CIBILTUSCR_PLSCORE = "CIBILTUIE1_CIBILTUSCR_PLSCORE"
    CIBILTUIE1_CIBILTUSC2_PLSCORE = "CIBILTUIE1_CIBILTUSC2_PLSCORE"
    CIBILTUIE1_CIBILTUSC3_PLSCORE = "CIBILTUIE1_CIBILTUSC3_PLSCORE"


class Gender(Enum):
    MALE = "MALE"
    FEMALE = "FEMALE"
    TRANSGENDER = "TRANSGENDER"


class IdentityType(Enum):
    PAN_CARD = "PAN_CARD"
    PASSPORT = "PASSPORT"
    VOTER_ID = "VOTER_ID"
    DRIVING_LICENSE = "DRIVING_LICENSE"
    RATION_CARD = "RATION_CARD"
    UID = "UID"


class PhoneNumberType(Enum):
    HOME = "HOME"
    OFFICE = "OFFICE"
    MOBILE = "MOBILE"
    NOT_CLASSIFIED = "NOT_CLASSIFIED"


class AddressType(Enum):
    PERMANENT = "PERMANENT"
    RESIDENCE = "RESIDENCE"
    OFFICE = "OFFICE"
    NOT_CATEGORIZED = "NOT_CATEGORIZED"


class ResidenceType(Enum):
    OWNED = "OWNED"
    RENTED = "RENTED"


class RelationType(Enum):
    FATHER = "FATHER"
    HUSBAND = "HUSBAND"
    MOTHER = "MOTHER"
    SON = "SON"
    DAUGHTER = "DAUGHTER"
    WIFE = "WIFE"
    BROTHER = "BROTHER"
    MOTHER_IN_LAW = "MOTHER_IN_LAW"
    FATHER_IN_LAW = "FATHER_IN_LAW"
    DAUGHTER_IN_LAW = "DAUGHTER_IN_LAW"
    SISTER_IN_LAW = "SISTER_IN_LAW"
    SON_IN_LAW = "SON_IN_LAW"
    BROTHER_IN_LAW = "BROTHER_IN_LAW"
    OTHER = "OTHER"



from unittest import mock
from unittest import TestCase

from mercuryclient.api import MercuryApi
from mercuryclient.types.cibil.request import CibilRequest, Applicant, Identity, Address


class CibilMixinTest(TestCase):
    def setUp(self):
        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        self.request_obj = CibilRequest(
            purpose="AUTO_LOAN",
            amount=100,
            gst_state="BIHAR",
            score_type="NO_SCORE",
            applicant=Applicant(
                first_name="Some",
                last_name="Name",
                date_of_birth="2000-01-01",
                gender="FEMALE",
                identities=[Identity(id_number="ABCDEFGH", id_type="PAN_CARD")],
                addresses=[
                    Address(
                        address_line_1="Random Address",
                        city="Random City",
                        pincode="123456",
                        state="BIHAR",
                    )
                ],
            ),
        )
        self.request_dict = {
            "profile": "some_profile",
            "purpose": "AUTO_LOAN",
            "amount": 100,
            "gst_state": "BIHAR",
            "score_type": "NO_SCORE",
            "applicant": {
                "first_name": "Some",
                "last_name": "Name",
                "date_of_birth": "2000-01-01",
                "gender": "FEMALE",
                "identities": [{"id_number": "ABCDEFGH", "id_type": "PAN_CARD"}],
                "addresses": [
                    {
                        "address_line_1": "Random Address",
                        "city": "Random City",
                        "pincode": "123456",
                        "state": "BIHAR",
                    }
                ],
            },
            "profile": "some_profile",
            "provider": None,
        }

        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)
        self.get_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._get_json_http_request"
        ).start()
        self.addCleanup(self.get_api_mock.stop)
        self.sleep_mock = mock.patch("mercuryclient.mixins.cibil.time.sleep").start()
        self.sleep_mock.return_value = None
        self.addCleanup(self.get_api_mock.stop)

    def test_request_cibil_report(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        self.client.request_cibil_report(self.request_obj, "some_profile", None)

        self.post_api_mock.assert_called_with(
            "api/v1/cibil/",
            data=self.request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.request_cibil_report(self.request_obj, "some_profile")

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        request_id = self.client.request_cibil_report(
            self.request_obj, "some_profile", None
        )
        self.assertEqual(request_id, "random_string")

    def test_response_cibil_report(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        self.get_api_mock.return_value = ("random_string", mock_response)
        self.client.get_cibil_response("random_string")

        self.get_api_mock.assert_called_with(
            "api/v1/cibil/",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )

    def test_response_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.get_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.get_cibil_response("random_string")

    def test_response_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        mock_response.json = mock.MagicMock()
        mock_response.json.return_value = {"status": "SUCCESS"}
        self.get_api_mock.return_value = ("random_string", mock_response)

        request_id, result = self.client.get_cibil_response("random_string")
        self.assertEqual(request_id, "random_string")
        self.assertEqual(result["status"], "SUCCESS")

    def test_entire_request_response_flow(self):
        mock_post_response = mock.MagicMock()
        mock_post_response.status_code = 201
        mock_get_response = mock.MagicMock()
        mock_get_response.status_code = 200
        mock_get_response.json = mock.MagicMock()
        mock_get_response.json.return_value = {"status": "SUCCESS"}
        self.post_api_mock.return_value = ("random_string", mock_post_response)
        self.get_api_mock.return_value = ("random_string", mock_get_response)

        response = self.client.fetch_cibil_report(self.request_obj, "some_profile")

        self.post_api_mock.assert_called_with(
            "api/v1/cibil/",
            data=self.request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )
        self.get_api_mock.assert_called_with(
            "api/v1/cibil/",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )
        self.sleep_mock.assert_called_with(15)
        self.assertEqual(response["status"], "SUCCESS")

    def test_entire_request_response_flow_failure(self):
        mock_post_response = mock.MagicMock()
        mock_post_response.status_code = 201
        mock_get_response = mock.MagicMock()
        mock_get_response.status_code = 200
        mock_get_response.json = mock.MagicMock()
        mock_get_response.json.return_value = {"status": "IN_PROGRESS"}
        self.post_api_mock.return_value = ("random_string", mock_post_response)
        self.get_api_mock.return_value = ("random_string", mock_get_response)

        with self.assertRaises(Exception):
            self.client.fetch_cibil_report(self.request_obj, "some_profile")



import time

import jwt

# Token validity threshold (in seconds). If token expires within the given
# threshold, we will get a new one access/refresh
REFRESH_TOKEN_THRESHOLD = 300
ACCESS_TOKEN_THRESHOLD = 120


class AuthMixin:
    """
    Mixin for client Authorization
    """

    def _authorize_client(self):
        """
        Authorize Mercury Client
        :return:
        """
        api = "api/v1/token/"

        if all([self.username, self.password]):
            data = {"username": self.username, "password": self.password}

            _, r = self._post_json_http_request(
                api, data=data, send_request_id=True, add_bearer_token=False
            )

            if r.status_code == 200:
                resp = r.json()
                self._access_token = resp["access"]
                self._refresh_token = resp["refresh"]
            else:
                raise Exception("Error Authorizing to Mercury")

    @staticmethod
    def is_token_valid(token, threshold):
        """
        Checks if a given token is valid past the threshold period
        :param token: string
        :param threshold: int
        :return:
        """
        decoded = jwt.decode(token, verify=False)
        return decoded["exp"] - int(time.time()) >= threshold

    def is_access_token_valid(self):
        if self._access_token:
            return self.is_token_valid(self._access_token, ACCESS_TOKEN_THRESHOLD)
        return False

    def is_refresh_token_valid(self):
        if self._refresh_token:
            return self.is_token_valid(self._refresh_token, REFRESH_TOKEN_THRESHOLD)
        return False

    @property
    def access_token(self):
        """
        Returns a valid access token
        :return:
        """
        if not self.is_access_token_valid():
            if self.is_refresh_token_valid():
                self.set_access_token()
            else:
                self._authorize_client()

        return self._access_token

    def set_access_token(self):
        """
        Sets the Access token using a valid refresh token
        :return:
        """
        api = "api/v1/token/refresh/"
        body = {"refresh": self._refresh_token}

        _, r = self._post_json_http_request(
            api, data=body, send_request_id=True, add_bearer_token=False
        )
        if r.status_code == 200:
            resp = r.json()
            self._access_token = resp["access"]
        else:
            raise Exception("Failed to get Access token")



from pydantic import BaseModel
from pydantic import Field


class AccountVerifyRequest(BaseModel):

    name: str = Field(max_length=50)
    phone: str = Field(max_length=10, min_length=10)
    bank_account_number: str = Field(max_length=50)
    bank_ifsc_code: str = Field(max_length=50)



from enum import Enum


class LegalConstitutionType(Enum):
    """
    Enums for ConstitutionType
    """

    SOLE_PROPRIETORSHIP = "SOLE_PROPRIETORSHIP"
    PARTNERSHIP_FIRM = "PARTNERSHIP_FIRM"
    HUF = "HUF"
    PRIVATE_LIMITED_COMPANY = "PRIVATE_LIMITED_COMPANY"
    PUBLIC_LIMITED_COMPANY = "PUBLIC_LIMITED_COMPANY"
    SOCIETY = "SOCIETY"
    ASSOCIATION_OF_PERSONS = "ASSOCIATION_OF_PERSONS"
    BODY_OF_INDIVIDUALS = "BODY_OF_INDIVIDUALS"
    TRUST = "TRUST"
    LIQUIDATOR = "LIQUIDATOR"
    LIMITED_LIABILITY_PARTNERSHIP = "LIMITED_LIABILITY_PARTNERSHIP"
    ARTIFICIAL_LIABILITY_PARTNERSHIP = "ARTIFICIAL_LIABILITY_PARTNERSHIP"
    PUBLIC_SECTOR_BANKS = "PUBLIC_SECTOR_BANKS"
    CENTRAL_OR_STATE_GOVERNMENT_DEPARTMENTS_OR_AGENCY = (
        "CENTRAL_OR_STATE_GOVERNMENT_DEPARTMENTS_OR_AGENCY"
    )
    SECTION_8_COMPANIES_ACT = "SECTION_8_COMPANIES_ACT"
    ARTIFICIAL_JURIDICAL_PERSON = "ARTIFICIAL_JURIDICAL_PERSON"
    INTERNATIONAL_ORGANISATION_OR_AGENCY = "INTERNATIONAL_ORGANISATION_OR_AGENCY"
    FOREIGN_EMBASSY = "FOREIGN_EMBASSY"
    CONSULAR_OFFICE = "CONSULAR_OFFICE"
    NOT_CATEGORIZED = "NOT_CATEGORIZED"
    OTHERS = "OTHERS"
    FOREIGN_PORTFOLIO_INVESTORS = "FOREIGN_PORTFOLIO_INVESTORS"


class AccountType(Enum):
    """
    Enums for AccountType
    """

    NORMAL = "NORMAL"
    SMALL = "SMALL"
    SIMPLIFIED = "SIMPLIFIED"
    OTPBASED = "OTPBASED"
    MINOR = "MINOR"


class IndividualRelatedPersonType(Enum):
    """
    Enums for RelatedPersonType
    """

    GUARDIAN_OF_MINOR = "GUARDIAN_OF_MINOR"
    ASSIGNEE = "ASSIGNEE"
    AUTHORIZED_REPRESENTATIVE = "AUTHORIZED_REPRESENTATIVE"


class LegalEntityRelatedPersonType(Enum):
    """
    Enums for RelatedPersonType
    """

    DIRECTOR = "DIRECTOR"
    PROMOTER = "PROMOTER"
    KARTA = "KARTA"
    TRUSTEE = "TRUSTEE"
    PARTNER = "PARTNER"
    PROPRIETOR = "PROPRIETOR"
    COURT_APPOINTED_OFFICIAL = "COURT_APPOINTED_OFFICIAL"
    BENEFICIARY = "BENEFICIARY"
    AUTHORISED_SIGNATORY = "AUTHORISED_SIGNATORY"
    BENEFICIAL_OWNER = "BENEFICIAL_OWNER"
    POWER_OF_ATTORNEY_HOLDER = "POWER_OF_ATTORNEY_HOLDER"
    OTHER = "OTHER"


class IndvIdentitiesType(Enum):
    """
    Enums for IdentitiesType
    """

    PASSPORT = "PASSPORT"
    VOTER_ID = "VOTER_ID"
    PAN = "PAN"
    DRIVING_LICENCE = "DRIVING_LICENCE"
    AADHAAR = "AADHAAR"
    NREGA_JOB_CARD = "NREGA_JOB_CARD"
    E_KYC_AUTHENTICATION = "E_KYC_AUTHENTICATION"
    OFFLINE_VERIFICATION_OF_AADHAAR = "OFFLINE_VERIFICATION_OF_AADHAAR"
    OTHERS = "OTHERS"


class LegalIdentitiesType(Enum):
    """
    Enums for Legal Identities Type
    """

    PAN = "PAN"
    OTHERS = "OTHERS"
    OFFICIALLY_VALID_DOCUMENT = "OFFICIALLY_VALID_DOCUMENT"
    CERTIFICATE_OF_INCORPORATION_OR_FORMATION = (
        "CERTIFICATE_OF_INCORPORATION_OR_FORMATION"
    )
    REGISTRATION_CERTIFICATE = "REGISTRATION_CERTIFICATE"
    MEMORANDUM_AND_ARTICLES_OF_ASSOCIATION = "MEMORANDUM_AND_ARTICLES_OF_ASSOCIATION"
    PARTNERSHIP_DEED = "PARTNERSHIP_DEED"
    TRUST_DEED = "TRUST_DEED"
    RESOLUTION_OF_BOARD_OR_MANAGING_COMMITTEE = (
        "RESOLUTION_OF_BOARD_OR_MANAGING_COMMITTEE"
    )
    POWER_OF_ATTORNEY_HOLDER = "POWER_OF_ATTORNEY_HOLDER"
    ACTIVITY_PROOF_1_FOR_SOLE_PROPRIETORSHIP_ONLY = (
        "ACTIVITY_PROOF_1_FOR_SOLE_PROPRIETORSHIP_ONLY"
    )
    ACTIVITY_PROOF_2_FOR_SOLE_PROPRIETORSHIP_ONLY = (
        "ACTIVITY_PROOF_2_FOR_SOLE_PROPRIETORSHIP_ONLY"
    )


class RelationIdentityType(Enum):
    """Enum RelationIdentityType"""

    AADHAAR_NUMBER = "AADHAAR_NUMBER"
    VOTER_ID_CARD = "VOTER_ID_CARD"
    NREGA_JOB_CARD = "NREGA_JOB_CARD"
    PASSPORT_NO = "PASSPORT_NO"
    DRIVING_LICENCE_NO = "DRIVING_LICENCE_NO"
    E_KYC = "E_KYC"
    OKYC = "OKYC"


class IdVerificationStatus(Enum):
    """
    Enums for IdentitiesType
    """

    No = "No"


class RelationshipType(Enum):
    """
    Enums for RelationshipType
    """

    FATHER = "FATHER"
    SPOUSE = "SPOUSE"


class Gender(Enum):
    """
    Enums for Gender
    """

    MALE = "MALE"
    FEMALE = "FEMALE"
    TRANSGENDER = "TRANSGENDER"


class CountryCode(Enum):
    """
    Enums for Country Code
    """

    AFGHANISTAN = "AFGHANISTAN"
    ALAND_ISLANDS_ = "ALAND_ISLANDS_"
    ALBANIA = "ALBANIA"
    ALGERIA = "ALGERIA"
    AMERICAN_SAMOA = "AMERICAN_SAMOA"
    ANDORRA = "ANDORRA"
    ANGOLA = "ANGOLA"
    ANGUILLA = "ANGUILLA"
    ANTARCTICA = "ANTARCTICA"
    ANTIGUA_AND_BARBUDA = "ANTIGUA_AND_BARBUDA"
    ARGENTINA = "ARGENTINA"
    ARMENIA = "ARMENIA"
    ARUBA = "ARUBA"
    AUSTRALIA = "AUSTRALIA"
    AUSTRIA = "AUSTRIA"
    AZERBAIJAN = "AZERBAIJAN"
    BAHAMAS = "BAHAMAS"
    BAHRAIN = "BAHRAIN"
    BANGLADESH = "BANGLADESH"
    BARBADOS = "BARBADOS"
    BELARUS = "BELARUS"
    BELGIUM = "BELGIUM"
    BELIZE = "BELIZE"
    BENIN = "BENIN"
    BERMUDA = "BERMUDA"
    BHUTAN = "BHUTAN"
    BOLIVIA_PLURINATIONAL_STATE = "BOLIVIA_PLURINATIONAL_STATE"
    BONAIRE_SINT_EUSTATIUS_AND_SABA = "BONAIRE_SINT_EUSTATIUS_AND_SABA"
    BOSNIA_AND_HERZEGOVINA = "BOSNIA_AND_HERZEGOVINA"
    BOTSWANA = "BOTSWANA"
    BOUVET_ISLAND = "BOUVET_ISLAND"
    BRAZIL = "BRAZIL"
    BRITISH_INDIAN_OCEAN_TERRITORY = "BRITISH_INDIAN_OCEAN_TERRITORY"
    BRUNEI_DARUSSALAM = "BRUNEI_DARUSSALAM"
    BULGARIA = "BULGARIA"
    BURKINA_FASO = "BURKINA_FASO"
    BURUNDI = "BURUNDI"
    CABO_VERDE = "CABO_VERDE"
    CAMBODIA = "CAMBODIA"
    CAMEROON = "CAMEROON"
    CANADA = "CANADA"
    CAYMAN_ISLANDS = "CAYMAN_ISLANDS"
    CENTRAL_AFRICAN_REPUBLIC = "CENTRAL_AFRICAN_REPUBLIC"
    CHAD = "CHAD"
    CHILE = "CHILE"
    CHINA = "CHINA"
    CHRISTMAS_ISLAND = "CHRISTMAS_ISLAND"
    COCOS_KEELING_ISLANDS = "COCOS_KEELING_ISLANDS"
    COLOMBIA = "COLOMBIA"
    COMOROS = "COMOROS"
    CONGO = "CONGO"
    CONGO_THE_DEMOCRATIC_REPUBLIC = "CONGO_THE_DEMOCRATIC_REPUBLIC"
    COOK_ISLANDS = "COOK_ISLANDS"
    COSTA_RICA = "COSTA_RICA"
    COTE_DIVOIRE = "COTE_DIVOIRE"
    CROATIA = "CROATIA"
    CUBA = "CUBA"
    CURACAO_CURAÇAO = "CURACAO_CURAÇAO"
    CYPRUS = "CYPRUS"
    CZECH_REPUBLIC = "CZECH_REPUBLIC"
    DENMARK = "DENMARK"
    DJIBOUTI = "DJIBOUTI"
    DOMINICA = "DOMINICA"
    DOMINICAN_REPUBLIC = "DOMINICAN_REPUBLIC"
    ECUADOR = "ECUADOR"
    EGYPT = "EGYPT"
    EL_SALVADOR = "EL_SALVADOR"
    EQUATORIAL_GUINEA = "EQUATORIAL_GUINEA"
    ERITREA = "ERITREA"
    ESTONIA = "ESTONIA"
    ETHIOPIA = "ETHIOPIA"
    FALKLAND_ISLANDS_MALVINAS = "FALKLAND_ISLANDS_MALVINAS"
    FAROE_ISLANDS = "FAROE_ISLANDS"
    FIJI = "FIJI"
    FINLAND = "FINLAND"
    FRANCE = "FRANCE"
    FRENCH_GUIANA = "FRENCH_GUIANA"
    FRENCH_POLYNESIA = "FRENCH_POLYNESIA"
    FRENCH_SOUTHERN_TERRITORIES = "FRENCH_SOUTHERN_TERRITORIES"
    GABON = "GABON"
    GAMBIA = "GAMBIA"
    GEORGIA = "GEORGIA"
    GERMANY = "GERMANY"
    GHANA = "GHANA"
    GIBRALTAR = "GIBRALTAR"
    GREECE = "GREECE"
    GREENLAND = "GREENLAND"
    GRENADA = "GRENADA"
    GUADELOUPE = "GUADELOUPE"
    GUAM = "GUAM"
    GUATEMALA = "GUATEMALA"
    GUERNSEY = "GUERNSEY"
    GUINEA = "GUINEA"
    GUINEABISSAU = "GUINEABISSAU"
    GUYANA = "GUYANA"
    HAITI = "HAITI"
    HEARD_ISLAND_AND_MCDONALD_ISLANDS = "HEARD_ISLAND_AND_MCDONALD_ISLANDS"
    HOLY_SEE_VATICAN_CITY_STATE = "HOLY_SEE_VATICAN_CITY_STATE"
    HONDURAS = "HONDURAS"
    HONG_KONG = "HONG_KONG"
    HUNGARY = "HUNGARY"
    ICELAND = "ICELAND"
    INDIA = "INDIA"
    INDONESIA = "INDONESIA"
    IRAN_ISLAMIC_REPUBLIC = "IRAN_ISLAMIC_REPUBLIC"
    IRAQ = "IRAQ"
    IRELAND = "IRELAND"
    ISLE_OF_MAN = "ISLE_OF_MAN"
    ISRAEL = "ISRAEL"
    ITALY = "ITALY"
    JAMAICA = "JAMAICA"
    JAPAN = "JAPAN"
    JERSEY = "JERSEY"
    JORDAN = "JORDAN"
    KAZAKHSTAN = "KAZAKHSTAN"
    KENYA = "KENYA"
    KIRIBATI = "KIRIBATI"
    KOREA_DEMOCRATIC_PEOPLE_REPUBLIC = "KOREA_DEMOCRATIC_PEOPLE_REPUBLIC"
    KOREA_REPUBLIC_OF = "KOREA_REPUBLIC_OF"
    KUWAIT = "KUWAIT"
    KYRGYZSTAN = "KYRGYZSTAN"
    LAO_PEOPLE_DEMOCRATIC_REPUBLIC = "LAO_PEOPLE_DEMOCRATIC_REPUBLIC"
    LATVIA = "LATVIA"
    LEBANON = "LEBANON"
    LESOTHO = "LESOTHO"
    LIBERIA = "LIBERIA"
    LIBYA = "LIBYA"
    LIECHTENSTEIN = "LIECHTENSTEIN"
    LITHUANIA = "LITHUANIA"
    LUXEMBOURG = "LUXEMBOURG"
    MACAO = "MACAO"
    MACEDONIA_THE_FORMER_YUGOSLAV_REPUBLIC = "MACEDONIA_THE_FORMER_YUGOSLAV_REPUBLIC"
    MADAGASCAR = "MADAGASCAR"
    MALAWI = "MALAWI"
    MALAYSIA = "MALAYSIA"
    MALDIVES = "MALDIVES"
    MALI = "MALI"
    MALTA = "MALTA"
    MARSHALL_ISLANDS = "MARSHALL_ISLANDS"
    MARTINIQUE = "MARTINIQUE"
    MAURITANIA = "MAURITANIA"
    MAURITIUS = "MAURITIUS"
    MAYOTTE = "MAYOTTE"
    MEXICO = "MEXICO"
    MICRONESIA_FEDERATED_STATES = "MICRONESIA_FEDERATED_STATES"
    MOLDOVA_REPUBLIC = "MOLDOVA_REPUBLIC"
    MONACO = "MONACO"
    MONGOLIA = "MONGOLIA"
    MONTENEGRO = "MONTENEGRO"
    MONTSERRAT = "MONTSERRAT"
    MOROCCO = "MOROCCO"
    MOZAMBIQUE = "MOZAMBIQUE"
    MYANMAR = "MYANMAR"
    NAMIBIA = "NAMIBIA"
    NAURU = "NAURU"
    NEPAL = "NEPAL"
    NETHERLANDS = "NETHERLANDS"
    NEW_CALEDONIA = "NEW_CALEDONIA"
    NEW_ZEALAND = "NEW_ZEALAND"
    NICARAGUA = "NICARAGUA"
    NIGER = "NIGER"
    NIGERIA = "NIGERIA"
    NIUE = "NIUE"
    NORFOLK_ISLAND = "NORFOLK_ISLAND"
    NORTHERN_MARIANA_ISLANDS = "NORTHERN_MARIANA_ISLANDS"
    NORWAY = "NORWAY"
    OMAN = "OMAN"
    PAKISTAN = "PAKISTAN"
    PALAU = "PALAU"
    PALESTINE_STATE_OF = "PALESTINE_STATE_OF"
    PANAMA = "PANAMA"
    PAPUA_NEW_GUINEA = "PAPUA_NEW_GUINEA"
    PARAGUAY = "PARAGUAY"
    PERU = "PERU"
    PHILIPPINES = "PHILIPPINES"
    PITCAIRN = "PITCAIRN"
    POLAND = "POLAND"
    PORTUGAL = "PORTUGAL"
    PUERTO_RICO = "PUERTO_RICO"
    QATAR = "QATAR"
    REUNION = "REUNION"
    ROMANIA = "ROMANIA"
    RUSSIAN_FEDERATION = "RUSSIAN_FEDERATION"
    RWANDA = "RWANDA"
    SAINT_BARTHELEMY = "SAINT_BARTHELEMY"
    SAINT_HELENA_ASCENSION_AND_TRISTAN_DA_CUNHA = (
        "SAINT_HELENA_ASCENSION_AND_TRISTAN_DA_CUNHA"
    )
    SAINT_KITTS_AND_NEVIS = "SAINT_KITTS_AND_NEVIS"
    SAINT_LUCIA = "SAINT_LUCIA"
    SAINT_MARTIN_FRENCH_PART = "SAINT_MARTIN_FRENCH_PART"
    SAINT_PIERRE_AND_MIQUELON = "SAINT_PIERRE_AND_MIQUELON"
    SAINT_VINCENT_AND_THE_GRENADINES = "SAINT_VINCENT_AND_THE_GRENADINES"
    SAMOA = "SAMOA"
    SAN_MARINO = "SAN_MARINO"
    SAO_TOME_AND_PRINCIPE = "SAO_TOME_AND_PRINCIPE"
    SAUDI_ARABIA = "SAUDI_ARABIA"
    SENEGAL = "SENEGAL"
    SERBIA = "SERBIA"
    SEYCHELLES = "SEYCHELLES"
    SIERRA_LEONE = "SIERRA_LEONE"
    SINGAPORE = "SINGAPORE"
    SINT_MAARTEN_DUTCH_PART = "SINT_MAARTEN_DUTCH_PART"
    SLOVAKIA = "SLOVAKIA"
    SLOVENIA = "SLOVENIA"
    SOLOMON_ISLANDS = "SOLOMON_ISLANDS"
    SOMALIA = "SOMALIA"
    SOUTH_AFRICA = "SOUTH_AFRICA"
    SOUTH_GEORGIA_AND_THE_SOUTH_SANDWICH_ISLANDS = (
        "SOUTH_GEORGIA_AND_THE_SOUTH_SANDWICH_ISLANDS"
    )
    SOUTH_SUDAN = "SOUTH_SUDAN"
    SPAIN = "SPAIN"
    SRI_LANKA = "SRI_LANKA"
    SUDAN = "SUDAN"
    SURINAME = "SURINAME"
    SVALBARD_AND_JAN_MAYEN = "SVALBARD_AND_JAN_MAYEN"
    SWAZILAND = "SWAZILAND"
    SWEDEN = "SWEDEN"
    SWITZERLAND = "SWITZERLAND"
    SYRIAN_ARAB_REPUBLIC = "SYRIAN_ARAB_REPUBLIC"
    TAIWAN_PROVINCE_OF_CHINA = "TAIWAN_PROVINCE_OF_CHINA"
    TAJIKISTAN = "TAJIKISTAN"
    TANZANIA_UNITED_REPUBLIC = "TANZANIA_UNITED_REPUBLIC"
    THAILAND = "THAILAND"
    TIMOR_LESTE = "TIMOR_LESTE"
    TOGO = "TOGO"
    TOKELAU = "TOKELAU"
    TONGA = "TONGA"
    TRINIDAD_AND_TOBAGO = "TRINIDAD_AND_TOBAGO"
    TUNISIA = "TUNISIA"
    TURKEY = "TURKEY"
    TURKMENISTAN = "TURKMENISTAN"
    TURKS_AND_CAICOS_ISLANDS = "TURKS_AND_CAICOS_ISLANDS"
    TUVALU = "TUVALU"
    UGANDA = "UGANDA"
    UKRAINE = "UKRAINE"
    UNITED_ARAB_EMIRATES = "UNITED_ARAB_EMIRATES"
    UNITED_KINGDOM = "UNITED_KINGDOM"
    UNITED_STATES = "UNITED_STATES"
    UNITED_STATES_MINOR_OUTLYING_ISLANDS = "UNITED_STATES_MINOR_OUTLYING_ISLANDS"
    URUGUAY = "URUGUAY"
    UZBEKISTAN = "UZBEKISTAN"
    VANUATU = "VANUATU"
    VENEZUELA_BOLIVARIAN_REPUBLIC_OF = "VENEZUELA_BOLIVARIAN_REPUBLIC_OF"
    VIET_NAM = "VIET_NAM"
    VIRGIN_ISLANDS_BRITISH = "VIRGIN_ISLANDS_BRITISH"
    VIRGIN_ISLANDS_U_S = "VIRGIN_ISLANDS_U_S"
    WALLIS_AND_FUTUNA = "WALLIS_AND_FUTUNA"
    WESTERN_SAHARA = "WESTERN_SAHARA"
    YEMEN = "YEMEN"
    ZAMBIA = "ZAMBIA"
    ZIMBABWE = "ZIMBABWE"


class StateMaster(Enum):
    """
    Enums for State Master
    """

    ANDAMAN_AND_NICOBAR_ISLANDS = "ANDAMAN_AND_NICOBAR_ISLANDS"
    ANDHRA_PRADESH = "ANDHRA_PRADESH"
    ARUNACHAL_PRADESH = "ARUNACHAL_PRADESH"
    ASSAM = "ASSAM"
    BIHAR = "BIHAR"
    CHANDIGARH = "CHANDIGARH"
    CHHATTISGARH = "CHHATTISGARH"
    DADRA_AND_NAGAR_HAVELI_ = "DADRA_AND_NAGAR_HAVELI_"
    DAMAN_AND_DIU = "DAMAN_AND_DIU"
    DELHI = "DELHI"
    GOA = "GOA"
    GUJARAT = "GUJARAT"
    HARYANA = "HARYANA"
    HIMACHAL_PRADESH = "HIMACHAL_PRADESH"
    JAMMU_AND_KASHMIR = "JAMMU_AND_KASHMIR"
    JHARKHAND = "JHARKHAND"
    KARNATAKA = "KARNATAKA"
    KERALA = "KERALA"
    LADAKH = "LADAKH"
    LAKSHADWEEP = "LAKSHADWEEP"
    MADHYA_PRADESH = "MADHYA_PRADESH"
    MAHARASHTRA = "MAHARASHTRA"
    MANIPUR = "MANIPUR"
    MEGHALAYA = "MEGHALAYA"
    MIZORAM = "MIZORAM"
    NAGALAND = "NAGALAND"
    ODISHA = "ODISHA"
    PUDUCHERRY = "PUDUCHERRY"
    PUNJAB = "PUNJAB"
    RAJASTHAN = "RAJASTHAN"
    SIKKIM = "SIKKIM"
    TAMIL_NADU = "TAMIL_NADU"
    TELANGANA = "TELANGANA"
    TRIPURA = "TRIPURA"
    UTTAR_PRADESH = "UTTAR_PRADESH"
    UTTARAKHAND = "UTTARAKHAND"
    WEST_BENGAL = "WEST_BENGAL"
    OTHER = "OTHER"
    ARMY_POST_OFFICE = "ARMY_POST_OFFICE"


class LegalAddressKyc(Enum):
    """
    Enums for Address KYC
    """

    CERTIFICATE_OF_INCORPORATION_OR_FORMATION = (
        "CERTIFICATE_OF_INCORPORATION_OR_FORMATION"
    )
    REGISTRATION_CERTIFICATE = "REGISTRATION_CERTIFICATE"


class IndvAddressKyc(Enum):
    """Enum for IndvAddressKyc"""

    PROOF_OF_POSSESSION_OF_AADHAAR = "PROOF_OF_POSSESSION_OF_AADHAAR"
    PASSPORT = "PASSPORT"
    DRIVING_LICENSE = "DRIVING_LICENSE"
    VOTERS_IDENTITY_CARD = "VOTERS_IDENTITY_CARD"
    NREGA_JOB_CARD = "NREGA_JOB_CARD"
    E_KYC_AUTHENTICATION = "E_KYC_AUTHENTICATION"
    OFFLINE_VERIFICATION_OF_AADHAAR = "OFFLINE_VERIFICATION_OF_AADHAAR"
    UTILITY_BILL = "UTILITY_BILL"
    PROPERTY_OR_MUNICIPAL_TAX_RECEIPT = "PROPERTY_OR_MUNICIPAL_TAX_RECEIPT"
    PENSION_OR_RETIRED_GOVERNMENT_DEPARTMENTS = (
        "PENSION_OR_RETIRED_GOVERNMENT_DEPARTMENTS"
    )
    SELF_DECLARATION = "SELF_DECLARATION"
    OTHER = "OTHER"


class KycTypes(Enum):
    """
    Enums for Kyc types
    """

    CERTIFIED_COPIES = "CERTIFIED_COPIES"
    EKYC = "EKYC"
    OKYC = "OKYC"
    DIGITAL_KYC = "DIGITAL_KYC"
    EQUIVALENT_E_DOCUMENT = "EQUIVALENT_E_DOCUMENT"
    VIDEO_KYC = "VIDEO_KYC"


class PhotographImageEnum(Enum):
    """
    Enums for Image Document Type
    """

    PHOTOGRAPH = "PHOTOGRAPH"


class IndvImageDocumentTypes(Enum):
    """
    Enums for Image Document Type
    """

    PAN = "PAN"
    PROOF_OF_POSSESSION_OF_AADHAAR = "PROOF_OF_POSSESSION_OF_AADHAAR"
    PASSPORT = "PASSPORT"
    DRIVING_LICENSE = "DRIVING_LICENSE"
    VOTERS_IDENTITY_CARD = "VOTERS_IDENTITY_CARD"
    NREGA_JOB_CARD = "NREGA_JOB_CARD"

    E_KYC_AUTHENTICATION = "E_KYC_AUTHENTICATION"
    OFFLINE_VERIFICATION_OF_AADHAAR = "OFFLINE_VERIFICATION_OF_AADHAAR"


class LegalImageDocumentTypes(Enum):
    """
    Enums for Image Document Type
    """

    OFFICIALLY_VALID_DOCUMENT_IN_RESPECT_OF_PERSON_AUTHORIZED_TO_TRANSACT = (
        "OFFICIALLY_VALID_DOCUMENT_IN_RESPECT_OF_PERSON_AUTHORIZED_TO_TRANSACT"
    )
    CERTIFICATE_OF_INCORPORATION_OR_FORMATION = (
        "CERTIFICATE_OF_INCORPORATION_OR_FORMATION"
    )
    REGISTRATION_CERTIFICATE = "REGISTRATION_CERTIFICATE"
    MEMORANDUM_AND_ARTICLES_OF_ASSOCIATION = "MEMORANDUM_AND_ARTICLES_OF_ASSOCIATION"
    PARTNERSHIP_DEED = "PARTNERSHIP_DEED"
    TRUST_DEED = "TRUST_DEED"
    RESOLUTION_OF_BOARD_OR_MANAGING_COMMITTEE = (
        "RESOLUTION_OF_BOARD_OR_MANAGING_COMMITTEE"
    )
    POWER_OF_ATTORNEY_HOLDER = "POWER_OF_ATTORNEY_HOLDER"
    ACTIVITY_PROOF_1_FOR_SOLE_PROPRIETORSHIP_ONLY = (
        "ACTIVITY_PROOF_1_FOR_SOLE_PROPRIETORSHIP_ONLY"
    )
    ACTIVITY_PROOF_2_FOR_SOLE_PROPRIETORSHIP_ONLY = (
        "ACTIVITY_PROOF_2_FOR_SOLE_PROPRIETORSHIP_ONLY"
    )
    OTHER = "OTHER"


class FlagType(Enum):
    """
    Enums for Flag
    """

    YES = "YES"
    NO = "NO"


class AddressType(Enum):
    """
    Enums for Address Type
    """

    PERMANENT = "PERMANENT"
    CURRENT = "CURRENT"


class IndvRelationPersonType(Enum):
    """
    Enums for Indv Relation Person Type
    """

    GUARDIAN_OF_MINOR = "GUARDIAN_OF_MINOR"
    ASSIGNEE = "ASSIGNEE"
    AUTHORISED_REPRESENTATIVE = "AUTHORISED_REPRESENTATIVE"


class LegalRelationPersonType(Enum):
    """
    Enums for Legal Relation Person Type
    """

    DIRECTOR = "DIRECTOR"
    PROMOTER = "PROMOTER"
    KARTA = "KARTA"
    TRUSTEE = "TRUSTEE"
    PARTNER = "PARTNER"
    PROPRIETOR = "PROPRIETOR"
    COURT_APPOINTED_OFFICIAL = "COURT_APPOINTED_OFFICIAL"
    BENEFICIARY = "BENEFICIARY"
    AUTHORISED_SIGNATORY = "AUTHORISED_SIGNATORY"
    BENEFICIAL_OWNER = "BENEFICIAL_OWNER"
    POWER_OF_ATTORNEY_HOLDER = "POWER_OF_ATTORNEY_HOLDER"
    OTHER = "OTHER"


class IdentitySearchTypes(Enum):
    """
    Enums for Identity Search type
    """

    VOTER_ID = "VOTER_ID"
    PASSPORT = "PASSPORT"
    PAN = "PAN"
    DRIVING_LICENSE = "DRIVING_LICENSE"
    PROOF_OF_POSSESSION_OF_AADHAAR = "PROOF_OF_POSSESSION_OF_AADHAAR"
    NREGA_JOB_CARD = "NREGA_JOB_CARD"
    NATIONAL_POPULATION_REGISTER_LETTER = "NATIONAL_POPULATION_REGISTER_LETTER"
    CERTIFICATE_OF_INCORPORATION = "CERTIFICATE_OF_INCORPORATION"
    CERTIFICATE_OF_FORMATION = "CERTIFICATE_OF_FORMATION"
    REGISTRATION_CERTIFICATE = "REGISTRATION_CERTIFICATE"



from enum import Enum


class EnquiryReason(Enum):
    AGRICULTURE_LOAN = "AGRICULTURE_LOAN"
    AUTO_LOAN = "AUTO_LOAN"
    BUSINESS_LOAN = "BUSINESS_LOAN"
    COMMERCIAL_VEHICLE_LOAN = "COMMERCIAL_VEHICLE_LOAN"
    CONSTRUCTION_EQUIPMENT_LOAN = "CONSTRUCTION_EQUIPMENT_LOAN"
    CONSUMER_LOAN = "CONSUMER_LOAN"
    CREDIT_CARD = "CREDIT_CARD"
    EDUCATION_LOAN = "EDUCATION_LOAN"
    LEASING = "LEASING"
    LOAN_AGAINST_COLLATERAL = "LOAN_AGAINST_COLLATERAL"
    MICROFINANCE = "MICROFINANCE"
    NON_FUNDED_CREDIT = "NON_FUNDED_CREDIT"
    PERSONAL_LOAN = "PERSONAL_LOAN"
    PROPERTY_LOAN = "PROPERTY_LOAN"
    TELECOM = "TELECOM"
    TWO_THREE_WHEELER_LOAN = "TWO_THREE_WHEELER_LOAN"
    WORKING_CAPITAL_LOAN = "WORKING_CAPITAL_LOAN"
    OTHERS = "OTHERS"


class FinancePurpose(Enum):
    AGRICULTURAL_MACHINERY = "AGRICULTURAL_MACHINERY"
    ANIMAL_HUSBANDRY = "ANIMAL_HUSBANDRY"
    AQUACULTURE = "AQUACULTURE"
    BIOGAS_PLANT = "BIOGAS_PLANT"
    CROP_LOAN = "CROP_LOAN"
    HORTICULTURE = "HORTICULTURE"
    IRRIGATION_SYSTEM = "IRRIGATION_SYSTEM"
    NEW_CAR = "NEW_CAR"
    OVERDRAFT_AGAINST_CAR = "OVERDRAFT_AGAINST_CAR"
    USED_CAR = "USED_CAR"
    GENERAL_BUSINESS = "GENERAL_BUSINESS"
    SMALL_MEDIUM_BUSINESS = "SMALL_MEDIUM_BUSINESS"
    PROFESSIONALS = "PROFESSIONALS"
    TRADE = "TRADE"
    BUS = "BUS"
    TEMPO = "TEMPO"
    TIPPER = "TIPPER"
    TRUCK = "TRUCK"
    TRACTOR = "TRACTOR"
    EXCAVATORS = "EXCAVATORS"
    FORKLIFT = "FORKLIFT"
    WHEEL_LOADERS = "WHEEL_LOADERS"
    CONSUMER_SEARCH = "CONSUMER_SEARCH"
    CREDIT_CARD = "CREDIT_CARD"
    FLEET_CARD = "FLEET_CARD"
    SECURED_CREDIT_CARD = "SECURED_CREDIT_CARD"
    CORPORATE_CREDIT_CARD = "CORPORATE_CREDIT_CARD"
    WORKING_EXECUTIVES = "WORKING_EXECUTIVES"
    STUDY_ABROAD = "STUDY_ABROAD"
    STUDY_INDIA = "STUDY_INDIA"
    LEASING = "LEASING"
    BANK_DEPOSITS = "BANK_DEPOSITS"
    GOLD = "GOLD"
    GOVT_BONDS = "GOVT_BONDS"
    SHARES_MUTUAL_FUNDS = "SHARES_MUTUAL_FUNDS"
    BUSINESS_LOAN = "BUSINESS_LOAN"
    HOUSING_LOAN = "HOUSING_LOAN"
    PERSONAL_LOAN = "PERSONAL_LOAN"
    AGRICULTURE = "AGRICULTURE"
    GENERAL_CREDIT = "GENERAL_CREDIT"
    SMALL_BUSINESS = "SMALL_BUSINESS"
    COMPUTERS = "COMPUTERS"
    CONSUMER_DURABLES = "CONSUMER_DURABLES"
    MARRIAGE_RELIGIOUS_CEREMONIES = "MARRIAGE_RELIGIOUS_CEREMONIES"
    TRAVEL = "TRAVEL"
    BALANCE_TRANSFER = "BALANCE_TRANSFER"
    HOME_IMPROVEMENT = "HOME_IMPROVEMENT"
    LAND = "LAND"
    LEASE_RENTAL_DISCOUNTING = "LEASE_RENTAL_DISCOUNTING"
    LOAN_AGAINST_PROPERTY = "LOAN_AGAINST_PROPERTY"
    NEW_HOME = "NEW_HOME"
    OFFICE_PREMISES = "OFFICE_PREMISES"
    UNDER_CONSTRUCTION = "UNDER_CONSTRUCTION"
    BROADBAND = "BROADBAND"
    LANDLINE = "LANDLINE"
    MOBILE = "MOBILE"
    THREE_WHEELER = "THREE_WHEELER"
    TWO_WHEELER = "TWO_WHEELER"
    CASH_CREDIT_FACILITY = "CASH_CREDIT_FACILITY"
    OVERDRAFT = "OVERDRAFT"
    TERM_LOAN = "TERM_LOAN"
    OTHERS = "OTHERS"


class Gender(Enum):
    MALE = "MALE"
    FEMALE = "FEMALE"
    TRANSGENDER = "TRANSGENDER"


class PhoneNumberType(Enum):
    HOME = "HOME"
    OFFICE = "OFFICE"
    MOBILE = "MOBILE"
    NOT_CLASSIFIED = "NOT_CLASSIFIED"


class MaritalStatus(Enum):
    SINGLE = "SINGLE"
    MARRIED = "MARRIED"
    DIVORCED = "DIVORCED"
    WIDOW_WIDOWER = "WIDOW_WIDOWER"


class EmploymentStatus(Enum):
    SALARIED = "SALARIED"
    NON_SALARIED = "NON_SALARIED"
    SELF_EMPLOYED = "SELF_EMPLOYED"
    SELF_EMPLOYED_PROFESSIONAL = "SELF_EMPLOYED_PROFESSIONAL"
    UNEMPLOYED = "UNEMPLOYED"


class State(Enum):
    ANDHRA_PRADESH = "ANDHRA_PRADESH"
    ARUNACHAL_PRADESH = "ARUNACHAL_PRADESH"
    ASSAM = "ASSAM"
    BIHAR = "BIHAR"
    GOA = "GOA"
    GUJARAT = "GUJARAT"
    HARYANA = "HARYANA"
    HIMACHAL_PRADESH = "HIMACHAL_PRADESH"
    JAMMU_KASHMIR = "JAMMU_KASHMIR"
    JHARKHAND = "JHARKHAND"
    KARNATAKA = "KARNATAKA"
    KERALA = "KERALA"
    MADHYA_PRADESH = "MADHYA_PRADESH"
    MAHARASHTRA = "MAHARASHTRA"
    MANIPUR = "MANIPUR"
    MEGHALAYA = "MEGHALAYA"
    MIZORAM = "MIZORAM"
    NAGALAND = "NAGALAND"
    ORISSA = "ORISSA"
    PUNJAB = "PUNJAB"
    RAJASTHAN = "RAJASTHAN"
    SIKKIM = "SIKKIM"
    TAMIL_NADU = "TAMIL_NADU"
    TELANGANA = "TELANGANA"
    TRIPURA = "TRIPURA"
    UTTAR_PRADESH = "UTTAR_PRADESH"
    UTTARANCHAL = "UTTARANCHAL"
    WEST_BENGAL = "WEST_BENGAL"
    ANDAMAN_NICOBAR_ISLANDS = "ANDAMAN_NICOBAR_ISLANDS"
    CHANDIGARH = "CHANDIGARH"
    CHATTISGARH = "CHATTISGARH"
    DADRA_NAGAR_HAVELI = "DADRA_NAGAR_HAVELI"
    DAMAN_DIU = "DAMAN_DIU"
    DELHI = "DELHI"
    LAKSHADWEEP = "LAKSHADWEEP"
    PONDICHERRY = "PONDICHERRY"



import logging

import requests

logger = logging.getLogger(__name__)


def log_response_time(r, *args, **kwargs):
    """
    Hook to log response time
    :param r:
    :param args:
    :param kwargs:
    :return:
    """
    # log in Milliseconds
    logger.info(
        "Mercury {},{},{},{}".format(
            r.request.method,
            r.url,
            r.status_code,
            int(r.elapsed.total_seconds() * 1000),
        )
    )


def get_session():
    """
    Build Mercury Session
    :return:
    """
    s = requests.Session()
    s.hooks["response"].append(log_response_time)
    return s



import json
from unittest import mock
from unittest import TestCase

from mercuryclient.api import MercuryApi
from mercuryclient.types.ckyc.request import IndividualEntityRequest, LegalEntityRequest


class CkycMixinTest(TestCase):
    def setUp(self):
        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )
        self.indv_data = {
            "fi_reference_no": "string",
            "account_type": "NORMAL",
            "personal_details": {
                "applicant_name": {
                    "name_prefix": "Mrs",
                    "first_name": "string",
                    "middle_name": "string",
                    "last_name": "string",
                },
                "applicant_maiden_name": {
                    "name_prefix": "Mrs",
                    "first_name": "string",
                    "middle_name": "string",
                    "last_name": "string",
                },
                "relationship_type": "FATHER",
                "relationship_person_name": {
                    "name_prefix": "Mrs",
                    "first_name": "string",
                    "middle_name": "string",
                    "last_name": "string",
                },
                "applicant_mother_name": {
                    "name_prefix": "Mrs",
                    "first_name": "string",
                    "middle_name": "string",
                    "last_name": "string",
                },
                "gender": "MALE",
                "date_of_birth": "2024-01-30",
                "pan_or_form_60": "string",
                "applicant_image": {
                    "image_url": "https://picsum.photos/id/2/5000/3333.jpg",
                    "image_type": "PHOTOGRAPH",
                },
            },
            "address_details": [
                {
                    "address_type": "CURRENT",
                    "address_line_1": "string",
                    "address_line_2": "string",
                    "address_line_3": "string",
                    "city": "string",
                    "district": "string",
                    "state": "ANDAMAN_AND_NICOBAR_ISLANDS",
                    "country": "AFGHANISTAN",
                    "pincode": "string",
                    "address_proof": "PROOF_OF_POSSESSION_OF_AADHAAR",
                    "address_proof_other_details": "string",
                    "id_proof_and_address_proof_same": True,
                    "address_proof_images": {
                        "image_url": "https://picsum.photos/id/3/5000/3333.jpg",
                        "image_type": "PAN",
                    },
                },
                {
                    "address_type": "PERMANENT",
                    "address_line_1": "string",
                    "address_line_2": "string",
                    "address_line_3": "string",
                    "city": "string",
                    "district": "string",
                    "state": "ANDAMAN_AND_NICOBAR_ISLANDS",
                    "country": "AFGHANISTAN",
                    "pincode": "string",
                    "address_proof": "PROOF_OF_POSSESSION_OF_AADHAAR",
                    "address_proof_other_details": "string",
                    "id_proof_and_address_proof_same": True,
                    "address_proof_images": {
                        "image_url": "https://picsum.photos/id/7/5000/3333.jpg",
                        "image_type": "PAN",
                    },
                },
            ],
            "contact_details": {
                "office_telephone_code": "+91",
                "office_telephone_no": "string",
                "mobile_code": "+91",
                "mobile_no": "0987654321",
                "email_id": "user@example.com",
                "residence_telephone_code": "+91",
                "residence_telephone_no": "string",
            },
            "remarks": "string",
            "kyc_verification_details": {
                "date_of_declaration": "2024-01-30",
                "place_of_declaration": "string",
                "kyc_verification_date": "2024-01-30",
                "type_of_document_submitted": "CERTIFIED_COPIES",
                "kyc_verification_name": "string",
                "kyc_verification_designation": "string",
                "kyc_verification_branch": "string",
                "kyc_verification_emp_code": "+91",
                "organisation_name": "string",
                "organisation_code": "+91",
            },
            "identity_details": [
                {
                    "identity_number": "string",
                    "other_identity_description": "string",
                    "identity_type": "PASSPORT",
                    "identity_image": {
                        "image_url": "https://picsum.photos/id/4/5000/3333.jpg",
                        "image_type": "PAN",
                    },
                }
            ],
            "relation_person_details": [
                {
                    "related_person_type_others_description": "string",
                    "ckyc_number": "string",
                    "name": {
                        "name_prefix": "Mrs",
                        "first_name": "string",
                        "middle_name": "string",
                        "last_name": "string",
                    },
                    "maiden_name": {
                        "name_prefix": "Mrs",
                        "first_name": "string",
                        "middle_name": "string",
                        "last_name": "string",
                    },
                    "relationship_type": "FATHER",
                    "relationship_person_name": {
                        "name_prefix": "Mrs",
                        "first_name": "string",
                        "middle_name": "string",
                        "last_name": "string",
                    },
                    "mother_name": {
                        "name_prefix": "Mrs",
                        "first_name": "string",
                        "middle_name": "string",
                        "last_name": "string",
                    },
                    "date_of_birth": "2024-01-30",
                    "gender": "MALE",
                    "nationality": "AFGHANISTAN",
                    "pan_or_form_60": "string",
                    "related_person_image": {
                        "image_url": "https://picsum.photos/id/5/5000/3333.jpg",
                        "image_type": "PHOTOGRAPH",
                    },
                    "contact_details": {
                        "office_telephone_code": "+91",
                        "office_telephone_no": "string",
                        "mobile_code": "+91",
                        "mobile_no": "9087654321",
                        "email_id": "user@example.com",
                    },
                    "kyc_verification_details": {
                        "date_of_declaration": "2024-01-30",
                        "place_of_declaration": "string",
                        "kyc_verification_date": "2024-01-30",
                        "type_of_document_submitted": "CERTIFIED_COPIES",
                        "kyc_verification_name": "string",
                        "kyc_verification_designation": "string",
                        "kyc_verification_branch": "string",
                        "kyc_verification_emp_code": "+91",
                        "organisation_name": "string",
                        "organisation_code": "+91",
                    },
                    "organisation_name": "string",
                    "organisation_code": "+91",
                    "remarks": "string",
                    "identity_details": {
                        "identity_number": "string",
                        "other_identity_description": "string",
                        "identity_type": "AADHAAR_NUMBER",
                        "identity_image": {
                            "image_url": "https://picsum.photos/id/0/5000/3333.jpg",
                            "image_type": "OFFICIALLY_VALID_DOCUMENT_IN_RESPECT_OF_PERSON_AUTHORIZED_TO_TRANSACT",
                        },
                    },
                    "address_details": [
                        {
                            "address_type": "PERMANENT",
                            "address_line_1": "string",
                            "address_line_2": "string",
                            "address_line_3": "string",
                            "city": "string",
                            "district": "string",
                            "state": "ANDAMAN_AND_NICOBAR_ISLANDS",
                            "country": "AFGHANISTAN",
                            "pincode": "string",
                            "address_proof": "PROOF_OF_POSSESSION_OF_AADHAAR",
                            "address_proof_other_details": "string",
                            "id_proof_and_address_proof_same": True,
                            "address_proof_images": {
                                "image_url": "https://picsum.photos/id/1/5000/3333.jpg",
                                "image_type": "PAN",
                            },
                        },
                        {
                            "address_type": "CURRENT",
                            "address_line_1": "string",
                            "address_line_2": "string",
                            "address_line_3": "string",
                            "city": "string",
                            "district": "string",
                            "state": "ANDAMAN_AND_NICOBAR_ISLANDS",
                            "country": "AFGHANISTAN",
                            "pincode": "string",
                            "address_proof": "PROOF_OF_POSSESSION_OF_AADHAAR",
                            "address_proof_other_details": "string",
                            "id_proof_and_address_proof_same": True,
                            "address_proof_images": {
                                "image_url": "https://picsum.photos/id/10/5000/3333.jpg",
                                "image_type": "PAN",
                            },
                        },
                    ],
                    "type_of_relationship": "GUARDIAN_OF_MINOR",
                }
            ],
        }
        self.legal_data = {
            "fi_reference_no": "string",
            "constitution_type": "SOLE_PROPRIETORSHIP",
            "constitution_type_others": "string",
            "legal_entity_details": {
                "name_of_the_entity": "string",
                "date_of_incorporation": "2024-07-24",
                "place_of_incorporation": "string",
                "date_of_commencement_of_business": "2024-07-24",
                "country_of_incorporation": "AFGHANISTAN",
                "tin_or_gst_registration_number": "string",
                "tin_issuing_country": "AFGHANISTAN",
                "pan_or_form_60": "string",
            },
            "address_details": [
                {
                    "address_type": "PERMANENT",
                    "address_line_1": "string",
                    "address_line_2": "string",
                    "address_line_3": "string",
                    "city": "string",
                    "district": "string",
                    "state": "ANDAMAN_AND_NICOBAR_ISLANDS",
                    "country": "AFGHANISTAN",
                    "pincode": "string",
                    "id_proof_and_address_proof_same": True,
                    "address_proof": "CERTIFICATE_OF_INCORPORATION_OR_FORMATION",
                    "address_proof_other_details": "string",
                    "address_proof_images": {
                        "image_url": "https://picsum.photos/id/0/5000/3333.jpg",
                        "image_type": "OFFICIALLY_VALID_DOCUMENT_IN_RESPECT_OF_PERSON_AUTHORIZED_TO_TRANSACT",
                    },
                }
            ],
            "contact_details": {
                "office_telephone_code": "str",
                "office_telephone_no": "string",
                "mobile_code": "str",
                "mobile_no": "9087654321",
                "email_id": "user@example.com",
                "fax_code": "str",
                "fax_no": "string",
                "alternate_email_id": "user@example.com",
                "alternate_mobile_number_code": "str",
                "alternate_mobile_number": "9087654321",
            },
            "remarks": "string",
            "kyc_verification_details": {
                "date_of_declaration": "2024-07-24",
                "place_of_declaration": "string",
                "kyc_verification_date": "2024-07-24",
                "type_of_document_submitted": "CERTIFIED_COPIES",
                "kyc_verification_name": "string",
                "kyc_verification_designation": "string",
                "kyc_verification_branch": "string",
                "kyc_verification_emp_code": "str",
                "organisation_name": "string",
                "organisation_code": "str",
            },
            "identity_details": [
                {
                    "identity_number": "string",
                    "other_identity_description": "string",
                    "identity_type": "PAN",
                    "identity_image": {
                        "image_url": "https://picsum.photos/id/1/5000/3333.jpg",
                        "image_type": "OFFICIALLY_VALID_DOCUMENT_IN_RESPECT_OF_PERSON_AUTHORIZED_TO_TRANSACT",
                    },
                }
            ],
            "relation_person_details": [
                {
                    "related_person_type_others_description": "string",
                    "ckyc_number": "string",
                    "name": {
                        "name_prefix": "str",
                        "first_name": "string",
                        "middle_name": "string",
                        "last_name": "string",
                    },
                    "maiden_name": {
                        "name_prefix": "str",
                        "first_name": "string",
                        "middle_name": "string",
                        "last_name": "string",
                    },
                    "relationship_type": "FATHER",
                    "relationship_person_name": {
                        "name_prefix": "str",
                        "first_name": "string",
                        "middle_name": "string",
                        "last_name": "string",
                    },
                    "mother_name": {
                        "name_prefix": "str",
                        "first_name": "string",
                        "middle_name": "string",
                        "last_name": "string",
                    },
                    "date_of_birth": "2024-07-24",
                    "gender": "MALE",
                    "nationality": "AFGHANISTAN",
                    "pan_or_form_60": "string",
                    "related_person_image": {
                        "image_url": "https://picsum.photos/id/2/5000/3333.jpg",
                        "image_type": "PHOTOGRAPH",
                    },
                    "contact_details": {
                        "office_telephone_code": "str",
                        "office_telephone_no": "string",
                        "mobile_code": "str",
                        "mobile_no": "9087654321",
                        "email_id": "user@example.com",
                    },
                    "kyc_verification_details": {
                        "date_of_declaration": "2024-07-24",
                        "place_of_declaration": "string",
                        "kyc_verification_date": "2024-07-24",
                        "type_of_document_submitted": "CERTIFIED_COPIES",
                        "kyc_verification_name": "string",
                        "kyc_verification_designation": "string",
                        "kyc_verification_branch": "string",
                        "kyc_verification_emp_code": "str",
                        "organisation_name": "string",
                        "organisation_code": "str",
                    },
                    "organisation_name": "string",
                    "organisation_code": "str",
                    "remarks": "string",
                    "identity_details": {
                        "identity_number": "string",
                        "other_identity_description": "string",
                        "identity_type": "AADHAAR_NUMBER",
                        "identity_image": {
                            "image_url": "https://picsum.photos/id/3/5000/3333.jpg",
                            "image_type": "OFFICIALLY_VALID_DOCUMENT_IN_RESPECT_OF_PERSON_AUTHORIZED_TO_TRANSACT",
                        },
                    },
                    "address_details": [
                        {
                            "address_type": "PERMANENT",
                            "address_line_1": "string",
                            "address_line_2": "string",
                            "address_line_3": "string",
                            "city": "string",
                            "district": "string",
                            "state": "ANDAMAN_AND_NICOBAR_ISLANDS",
                            "country": "AFGHANISTAN",
                            "pincode": "string",
                            "id_proof_and_address_proof_same": True,
                            "address_proof": "CERTIFICATE_OF_INCORPORATION_OR_FORMATION",
                            "address_proof_other_details": "string",
                            "address_proof_images": {
                                "image_url": "https://picsum.photos/id/4/5000/3333.jpg",
                                "image_type": "OFFICIALLY_VALID_DOCUMENT_IN_RESPECT_OF_PERSON_AUTHORIZED_TO_TRANSACT",
                            },
                        }
                    ],
                    "type_of_relationship": "GUARDIAN_OF_MINOR",
                    "director_identification_number": "string",
                }
            ],
        }
        self.indv_request_obj = IndividualEntityRequest(**self.indv_data)
        self.legal_request_obj = LegalEntityRequest(**self.legal_data)
        self.indv_request_dict = {
            **json.loads(self.indv_request_obj.json(exclude_unset=True)),
            **{"profile": "some_profile"},
        }
        self.legal_request_dict = {
            **json.loads(self.legal_request_obj.json(exclude_unset=True)),
            **{"profile": "some_profile"},
        }
        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)
        self.get_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._get_json_http_request"
        ).start()
        self.addCleanup(self.get_api_mock.stop)
        self.sleep_mock = mock.patch("mercuryclient.mixins.ckyc.time.sleep").start()
        self.sleep_mock.return_value = None
        self.addCleanup(self.get_api_mock.stop)

    def test_request_ckyc_report(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_data = json.loads(self.indv_request_obj.json(exclude_unset=True))
        request_data["profile"] = "some_profile"
        self.client._initiate_ckyc_request(
            api="ckyc/indv/entity/upload/", request_dict=request_data
        )
        self.post_api_mock.assert_called_with(
            "api/v1/ckyc/indv/entity/upload/",
            data=self.indv_request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_data = json.loads(self.indv_request_obj.json(exclude_unset=True))
        request_data["profile"] = "some_profile"
        with self.assertRaises(Exception):
            self.client._initiate_ckyc_request(request_data)

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        request_data = json.loads(self.indv_request_obj.json(exclude_unset=True))
        request_data["profile"] = "some_profile"
        request_id = self.client._initiate_ckyc_request("some_profile", request_data)
        self.assertEqual(request_id, "random_string")

    def test_response_ckyc_report(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        self.get_api_mock.return_value = ("random_string", mock_response)
        self.client._get_ckyc_response("random_string")

        self.get_api_mock.assert_called_with(
            "api/v1/ckyc/result",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )

    def test_response_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.get_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.fetch_ckyc_result("random_string")

    def test_response_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        mock_response.json = mock.MagicMock()
        mock_response.json.return_value = {"status": "SUCCESS"}
        self.get_api_mock.return_value = ("random_string", mock_response)

        request_id, result = self.client._get_ckyc_response("random_string")
        self.assertEqual(request_id, "random_string")
        self.assertEqual(result["status"], "SUCCESS")

    def test_entire_request_response_flow_indv(self):
        mock_post_response = mock.MagicMock()
        mock_post_response.status_code = 201
        mock_get_response = mock.MagicMock()
        mock_get_response.status_code = 200
        mock_get_response.json = mock.MagicMock()
        mock_get_response.json.return_value = {"status": "SUCCESS"}
        self.post_api_mock.return_value = ("random_string", mock_post_response)
        self.get_api_mock.return_value = ("random_string", mock_get_response)

        response = self.client.create_indv_entity("some_profile", self.indv_request_obj)

        self.post_api_mock.assert_called_with(
            "api/v1/ckyc/indv/entity/upload/",
            data=self.indv_request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )
        self.get_api_mock.assert_called_with(
            "api/v1/ckyc/result",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )
        self.sleep_mock.assert_called_with(15)
        self.assertEqual(response["status"], "SUCCESS")

    def test_entire_request_response_flow_failure_indv(self):
        mock_post_response = mock.MagicMock()
        mock_post_response.status_code = 201
        mock_get_response = mock.MagicMock()
        mock_get_response.status_code = 200
        mock_get_response.json = mock.MagicMock()
        mock_get_response.json.return_value = {"status": "IN_PROGRESS"}
        self.post_api_mock.return_value = ("random_string", mock_post_response)
        self.get_api_mock.return_value = ("random_string", mock_get_response)

        with self.assertRaises(Exception):
            self.client.create_indv_entity("some_profile", self.indv_request_obj)

    def test_entire_request_response_flow_legal(self):
        mock_post_response = mock.MagicMock()
        mock_post_response.status_code = 201
        mock_get_response = mock.MagicMock()
        mock_get_response.status_code = 200
        mock_get_response.json = mock.MagicMock()
        mock_get_response.json.return_value = {"status": "SUCCESS"}
        self.post_api_mock.return_value = ("random_string", mock_post_response)
        self.get_api_mock.return_value = ("random_string", mock_get_response)

        response = self.client.create_legal_entity(
            "some_profile", self.legal_request_obj
        )

        self.post_api_mock.assert_called_with(
            "api/v1/ckyc/legal/entity/upload/",
            data=self.legal_request_dict,
            send_request_id=True,
            add_bearer_token=True,
        )
        self.get_api_mock.assert_called_with(
            "api/v1/ckyc/result",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )
        self.sleep_mock.assert_called_with(15)
        self.assertEqual(response["status"], "SUCCESS")

    def test_entire_request_response_flow_failure_legal(self):
        mock_post_response = mock.MagicMock()
        mock_post_response.status_code = 201
        mock_get_response = mock.MagicMock()
        mock_get_response.status_code = 200
        mock_get_response.json = mock.MagicMock()
        mock_get_response.json.return_value = {"status": "IN_PROGRESS"}
        self.post_api_mock.return_value = ("random_string", mock_post_response)
        self.get_api_mock.return_value = ("random_string", mock_get_response)

        with self.assertRaises(Exception):
            self.client.create_legal_entity("some_profile", self.legal_request_dict)



import time
from typing import Tuple


class RekognitionMixins:
    """
    Mixin for face rekognition
    """

    def generate_liveness_session_id(
        self, profile, provider, token, liveness_prefix="image_auth"
    ) -> Tuple[str, dict]:
        api = "api/v1/rekognition/create/session"
        payload = dict(
            profile=profile,
            provider=provider,
            token=token,
            liveness_prefix=liveness_prefix,
        )

        request_id, response = self._post_json_http_request(
            api, data=payload, send_request_id=True, add_bearer_token=True
        )

        if response.status_code == 201:
            result = response.json()
            return request_id, result

        try:
            response_json = response.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while generating Session id for rekognition. Status: {}, Response is {}".format(
                response.status_code, response_json
            )
        )

    def get_liveness_session_result(
        self, profile: str, provider: str, session_id: str
    ) -> Tuple[str, dict]:
        api = "api/v1/rekognition/verify/session"
        payload = dict(profile=profile, provider=provider, session_id=session_id)
        request_id, response = self._post_json_http_request(
            api, data=payload, send_request_id=True, add_bearer_token=True
        )

        if response.status_code == 200:
            result = response.json()
            return request_id, result

        try:
            response_json = response.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error getting rekognition session result. Status: {}, Response is {}".format(
                response.status_code, response_json
            )
        )



import time
from typing import Tuple


class ItrVerificationMixin:
    """
    Mixin for verifying itr
    """

    def initiate_itr_pull_request(
        self,
        request_data,
    ) -> str:
        api = "api/v1/itr/pull"
        request_id, r = self._post_json_http_request(
            api, data=request_data, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:
            return request_id

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending itr verification request. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def get_itr_response(self, request_id: str) -> Tuple[str, dict]:
        api = "api/v1/itr/pull"

        request_id, r = self._get_json_http_request(
            api,
            headers={"X-Mercury-Request-Id": request_id},
            send_request_id=False,
            add_bearer_token=True,
        )

        if r.status_code == 200:
            result = r.json()
            return request_id, result

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error getting itr Verification result. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def fetch_itr_report(
        self,
        profile: str,
        provider: str,
        username: str,
        password: bool,
        max_attempts: int = 3,
        retry_backoff: int = 5,
    ):
        """Based on itr it fetches all the details associated with it
        returns: mercury_request_id, verify itr response from vendor
        """
        request_data = {
            "provider": provider,
            "profile": profile,
            "username": username,
            "password": password,
        }

        mercury_request_id = self.initiate_itr_pull_request(request_data)

        for attempts in range(max_attempts):
            time.sleep(retry_backoff)
            mercury_request_id, result = self.get_itr_response(mercury_request_id)
            if result.get("status") != "IN_PROGRESS":
                return mercury_request_id, result

            retry_backoff *= 2

        raise Exception("Error while getting itr response. Status: IN_PROGRESS")



from datetime import date
from unittest import TestCase, mock

from mercuryclient.api import MercuryApi


class OkycVerificationMixinTest(TestCase):
    def setUp(self):
        self.client = MercuryApi(
            {
                "username": "username",
                "password": "password",
                "url": "https://mercury-dev.esthenos.in",
            }
        )

        self.post_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._post_json_http_request"
        ).start()
        self.addCleanup(self.post_api_mock.stop)
        self.get_api_mock = mock.patch(
            "mercuryclient.api.MercuryApi._get_json_http_request"
        ).start()
        self.addCleanup(self.get_api_mock.stop)
        self.request_data = {
            "request_type": "OTP_REQUEST",
            "provider": "zoop",
            "profile": "some_profile",
            "id_number": "ABCDE1234",
        }

    def test_wrong_request_type_raises_exception(self):
        with self.assertRaises(Exception) as exc:
            self.request_data["request_type"] = "Something"
            self.client.initiate_okyc_request(self.request_data)

        self.assertEqual(str(exc.exception), "Something is not a valid request_type")
        self.post_api_mock.assert_not_called()

    def test_request_okyc_verification(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)
        self.client.initiate_okyc_request(self.request_data)

        self.post_api_mock.assert_called_with(
            "api/v1/okyc_verification/",
            data={
                "provider": "zoop",
                "profile": "some_profile",
                "request_type": "OTP_REQUEST",
                "id_number": "ABCDE1234",
            },
            send_request_id=True,
            add_bearer_token=True,
        )

    def test_request_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.post_api_mock.return_value = ("random_string", mock_response)

        with self.assertRaises(Exception):
            self.client.initiate_okyc_request(self.request_data)

    def test_request_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 201
        self.post_api_mock.return_value = ("random_string", mock_response)

        request_id = self.client.initiate_okyc_request(self.request_data)
        self.assertEqual(request_id, "random_string")

    def test_response_verify_okyc(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        self.get_api_mock.return_value = ("random_string", mock_response)
        self.client.get_okyc_response("random_string")

        self.get_api_mock.assert_called_with(
            "api/v1/okyc_verification/",
            headers={"X-Mercury-Request-Id": "random_string"},
            send_request_id=False,
            add_bearer_token=True,
        )

    def test_response_exception_raised_if_status_code_error(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 401
        self.get_api_mock.return_value = ("random_string", mock_response)
        with self.assertRaises(Exception):
            self.client.get_okyc_response("random_string")

    def test_response_api_succeeds_if_status_code_success(self):
        mock_response = mock.MagicMock()
        mock_response.status_code = 200
        mock_response.json = mock.MagicMock()
        mock_response.json.return_value = {"status": "SUCCESS"}
        self.get_api_mock.return_value = ("random_string", mock_response)

        request_id, result = self.client.get_okyc_response("random_string")
        self.assertEqual(request_id, "random_string")
        self.assertEqual(result["status"], "SUCCESS")



import time
import json
from typing import Tuple

from mercuryclient.types.ckyc.request import IndividualEntityRequest, LegalEntityRequest


class CkycMixin:
    """
    Mixin for registering Ckyc requests
    """

    def _initiate_ckyc_request(
        self,
        api: str,
        request_dict: dict,
    ) -> str:
        """
        POST a request to Mercury to Request Ckyc report. This posts the request to
        Mercury and returns immediately. Use the returned request ID and poll
        _get_ckyc_response to check for report.
        You can also use fetch_Ckyc_report which is a helper function that combines
        this api and the result api to get the result.

        :param request_obj: Object of CkycRequest model
        :type request_obj: CkycRequest
        :param profile: Ckyc profile name
        :type profile: str
        :return: Dict containing request ID and status
        :rtype: dict
        """
        api = "api/v1/" + api
        request_id, r = self._post_json_http_request(
            api, data=request_dict, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 201:
            return request_id

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending Ckyc request. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def _get_ckyc_response(self, request_id: str) -> Tuple[str, dict]:
        """
        Get result for ckyc job request at the provided request ID. You can poll
        this method to check for response. If the job is still progressing, the status
        within the response will be IN_PROGRESS and the method can continue to be polled
        until you get either a SUCCESS or FAILURE

        :param request_id: Request ID of the CKYC job request
        :type request_id: str
        :return: Tuple containing request ID and job response. This response will
            contain the "status" of the job (IN_PROGRESS, SUCCESS or FAILURE), a
            "message" and a "data" key containing a dict with the bureau report
        :rtype: Tuple[str, dict]
        """
        api = "api/v1/ckyc/result"

        request_id, r = self._get_json_http_request(
            api,
            headers={"X-Mercury-Request-Id": request_id},
            send_request_id=False,
            add_bearer_token=True,
        )

        if r.status_code == 200:
            return request_id, r.json()

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while getting Ckyc response. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )

    def create_indv_entity(
        self,
        profile: str,
        request_obj: IndividualEntityRequest,
        max_attempts: int = 8,
        retry_backoff: int = 15,
    ):
        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile

        request_id = self._initiate_ckyc_request(
            "ckyc/indv/entity/upload/", request_dict
        )

        return self.fetch_ckyc_result(request_id, max_attempts, retry_backoff)

    def create_legal_entity(
        self,
        profile: str,
        request_obj: LegalEntityRequest,
        max_attempts: int = 8,
        retry_backoff: int = 15,
    ):
        request_dict = json.loads(request_obj.json(exclude_unset=True))
        request_dict["profile"] = profile

        request_id = self._initiate_ckyc_request(
            "ckyc/legal/entity/upload/", request_dict
        )

        return self.fetch_ckyc_result(request_id, max_attempts, retry_backoff)

    def fetch_ckyc_result(
        self, request_id: str, max_attempts: int = 8, retry_backoff: int = 15
    ):
        attempts = 0
        while attempts < max_attempts:
            time.sleep(retry_backoff)
            request_id, result = self._get_ckyc_response(request_id)
            if result.get("status") != "IN_PROGRESS":
                return result

            retry_backoff *= 2
            attempts += 1

        raise Exception("Error while getting Ckyc response. Status: IN_PROGRESS")



from urllib.parse import urlparse
import PyPDF2
import requests
from io import BytesIO


class ESignVerification:
    """ """

    def _download_pdf_from_url(self, pdf_url):
        response = requests.get(pdf_url)
        if response.status_code != 200:
            return None
        content_type = response.headers.get("Content-Type", "")
        if "application/pdf" in content_type or response.content.startswith(b"%PDF"):
            return BytesIO(response.content)

    def check_e_sign(self, profile, provider, pdf_path):
        # Open the PDF file
        file_content = self._download_pdf_from_url(pdf_path)
        parsed_url = urlparse(pdf_path)
        file_name = parsed_url.path.split("/")[-1]
        if not file_content:
            raise Exception("Document file must be a PDF")
        # Create a PDF reader object
        reader = PyPDF2.PdfFileReader(file_content)
        # Check if the document has signature fields
        signature_count = 0
        number_of_pages = len(reader.pages)
        sign_per_page = {}
        for page_no, page in enumerate(reader.pages, start=1):
            per_page_sign = 0
            fields = page.get("/Annots")
            if fields:
                for field in fields:
                    field_obj = field.getObject()
                    if field_obj.get("/FT") == "/Sig":
                        signature_count += 1
                        per_page_sign += 1
            sign_per_page[f"page_{page_no}"] = per_page_sign

        api = "api/v1/e_sign/verify/"
        request_data = dict(
            profile=profile,
            provider=provider,
            signature_count=signature_count,
            file_name=file_name,
            number_of_pages=number_of_pages,
            sign_per_page=sign_per_page,
        )
        request_id, r = self._post_json_http_request(
            api, data=request_data, send_request_id=True, add_bearer_token=True
        )

        if r.status_code == 200:
            return request_id, r.json()

        try:
            response_json = r.json()
        except Exception:
            response_json = {}
        raise Exception(
            "Error while sending e-signature request. Status: {}, Response is {}".format(
                r.status_code, response_json
            )
        )



from datetime import date

from pydantic import BaseModel

from .enums import Countries, Gender, PassportTypes


class PassportDetails(BaseModel):
    first_name: str
    last_name: str
    date_of_birth: date
    expiry_date: date
    gender: Gender
    passport_type: PassportTypes
    country: Countries



from datetime import date
from typing import List, Optional
from pydantic import BaseModel, Field
from pydantic import HttpUrl, conlist

from mercuryclient.types.ckyc.enums import (
    AccountType,
    AddressType,
    CountryCode,
    Gender,
    IndvAddressKyc,
    IndvIdentitiesType,
    IndvImageDocumentTypes,
    IndvRelationPersonType,
    KycTypes,
    LegalAddressKyc,
    LegalConstitutionType,
    LegalIdentitiesType,
    LegalImageDocumentTypes,
    PhotographImageEnum,
    RelationIdentityType,
    RelationshipType,
    StateMaster,
)
from mercuryclient.types.common import NormalizedString


class ImageDetails(BaseModel):
    """
    Image Details
    """

    image_url: HttpUrl


class PhotographImage(ImageDetails):
    """
    Image Details
    """

    image_type: PhotographImageEnum


class IndvImageDetails(ImageDetails):
    """
    Image Details
    """

    image_type: IndvImageDocumentTypes


class LegalImageDetails(ImageDetails):
    """
    Image Details
    """

    image_type: LegalImageDocumentTypes


class NameDetails(BaseModel):
    """
    Name
    """

    name_prefix: NormalizedString = Field(min_length=2, max_length=5)
    first_name: NormalizedString = Field(min_length=3, max_length=50)
    middle_name: Optional[NormalizedString] = Field(min_length=3, max_length=50)
    last_name: Optional[NormalizedString] = Field(min_length=3, max_length=50)


class IndvPersonalDetails(BaseModel):
    """
    personal details
    """

    applicant_name: NameDetails
    applicant_maiden_name: Optional[NameDetails]
    relationship_type: Optional[RelationshipType]
    relationship_person_name: Optional[NameDetails]
    applicant_mother_name: Optional[NameDetails]
    gender: Gender
    date_of_birth: date
    pan_or_form_60: NormalizedString
    applicant_image: PhotographImage


class AddressDetails(BaseModel):

    """
    Address
    """

    address_type: AddressType
    address_line_1: NormalizedString = Field(max_length=55)
    address_line_2: Optional[NormalizedString] = Field(max_length=55)
    address_line_3: Optional[NormalizedString] = Field(max_length=55)
    city: NormalizedString = Field(max_length=50)
    district: NormalizedString = Field(max_length=50)
    state: StateMaster
    country: CountryCode
    pincode: Optional[NormalizedString] = Field(max_length=10)
    id_proof_and_address_proof_same: bool


class IndvAddress(AddressDetails):
    """
    IndvAddress
    """

    address_proof: IndvAddressKyc
    address_proof_other_details: Optional[NormalizedString] = Field(max_length=75)
    address_proof_images: Optional[IndvImageDetails]


class LegalAddress(AddressDetails):
    """
    LegalAddress
    """

    address_proof: LegalAddressKyc
    address_proof_other_details: Optional[NormalizedString] = Field(
        max_length=75,
    )
    address_proof_images: Optional[LegalImageDetails]


class ContactDetails(BaseModel):
    """ContactDetails"""

    office_telephone_code: Optional[NormalizedString] = Field(max_length=4)
    office_telephone_no: Optional[NormalizedString] = Field(max_length=10)
    mobile_code: Optional[NormalizedString] = Field(max_length=3)
    mobile_no: Optional[NormalizedString] = Field(min_length=10, max_length=20)
    email_id: Optional[NormalizedString]


class IndvContactDetails(ContactDetails):

    """
    Contact
    """

    residence_telephone_code: Optional[NormalizedString] = Field(max_length=4)
    residence_telephone_no: Optional[NormalizedString] = Field(max_length=10)


class LegalContactDetails(ContactDetails):

    """
    Contact
    """

    fax_code: Optional[NormalizedString] = Field(max_length=4)
    fax_no: Optional[NormalizedString] = Field(max_length=10)
    alternate_email_id: Optional[NormalizedString]
    alternate_mobile_number_code: Optional[NormalizedString] = Field(max_length=3)
    alternate_mobile_number: Optional[NormalizedString] = Field(
        min_length=10, max_length=20
    )


class KycDetails(BaseModel):
    """
    Kyc
    """

    date_of_declaration: date
    place_of_declaration: NormalizedString = Field(max_length=50)
    kyc_verification_date: date
    type_of_document_submitted: KycTypes
    kyc_verification_name: NormalizedString = Field(max_length=150)
    kyc_verification_designation: NormalizedString = Field(max_length=50)
    kyc_verification_branch: NormalizedString = Field(max_length=50)
    kyc_verification_emp_code: NormalizedString = Field(max_length=50)
    organisation_name: NormalizedString = Field(max_length=150)
    organisation_code: NormalizedString = Field(max_length=7)


class IdentityDetails(BaseModel):
    """
    Identity Details
    """

    identity_number: NormalizedString = Field(max_length=20)
    other_identity_description: Optional[NormalizedString]


class IndvIdentity(IdentityDetails):
    """IndvIdentity"""

    identity_type: IndvIdentitiesType
    identity_image: IndvImageDetails


class LegalIdentity(IdentityDetails):
    """LegalIdentity"""

    identity_type: LegalIdentitiesType
    identity_image: LegalImageDetails


class RelatedIdentity(IdentityDetails):
    """
    Related Identity Details
    """

    identity_type: RelationIdentityType
    identity_image: LegalImageDetails


class RelationPersonDetails(BaseModel):
    """
    Relation Person Details
    """

    related_person_type_others_description: NormalizedString = Field(
        max_length=150, required=False
    )

    ckyc_number: NormalizedString = Field(max_length=14, required=False)
    name: NameDetails
    maiden_name: Optional[NameDetails]
    relationship_type: Optional[RelationshipType]
    relationship_person_name: Optional[NameDetails]
    mother_name: Optional[NameDetails]
    date_of_birth: date
    gender: Gender
    nationality: CountryCode
    pan_or_form_60: NormalizedString = Field(min_length=3, max_length=10)
    related_person_image: PhotographImage
    contact_details: Optional[ContactDetails]
    kyc_verification_details: KycDetails
    remarks: Optional[NormalizedString] = Field(default="", max_length=300)
    identity_details: RelatedIdentity


class IndvRelationPerson(RelationPersonDetails):
    """IndvRelationPerson"""

    address_details: conlist(IndvAddress, max_items=2)  # type: ignore
    type_of_relationship: IndvRelationPersonType


class LegalRelationPerson(RelationPersonDetails):
    """LegalRelationPerson"""

    address_details: conlist(LegalAddress, max_items=2)  # type: ignore
    type_of_relationship: IndvRelationPersonType
    director_identification_number: Optional[NormalizedString] = Field(max_length=8)


class IndividualEntityRequest(BaseModel):

    fi_reference_no: NormalizedString = Field(min_length=3, max_length=20)
    account_type: AccountType
    personal_details: IndvPersonalDetails
    address_details: conlist(IndvAddress, max_items=2)  # type: ignore
    contact_details: Optional[IndvContactDetails]
    remarks: Optional[NormalizedString] = Field(default="", max_length=300)
    kyc_verification_details: KycDetails
    identity_details: conlist(IndvIdentity, min_items=1)  # type: ignore
    relation_person_details: Optional[List[IndvRelationPerson]]


class LegalEntityDetails(BaseModel):
    """
    personal details
    """

    name_of_the_entity: Optional[NormalizedString] = Field(
        min_length=3,
        max_length=150,
    )

    date_of_incorporation: date
    place_of_incorporation: Optional[NormalizedString] = Field(
        min_length=3,
        max_length=50,
    )

    date_of_commencement_of_business: Optional[date]
    country_of_incorporation: Optional[CountryCode]

    tin_or_gst_registration_number: Optional[NormalizedString] = Field(
        min_length=3,
        max_length=20,
    )
    tin_issuing_country: CountryCode
    pan_or_form_60: NormalizedString = Field(min_length=3, max_length=10)


class LegalEntityRequest(BaseModel):
    """LegalEntityRequest"""

    fi_reference_no: NormalizedString = Field(min_length=3, max_length=20)
    constitution_type: LegalConstitutionType
    constitution_type_others: NormalizedString = Field(
        default="", min_length=3, max_length=20
    )
    legal_entity_details: LegalEntityDetails
    address_details: conlist(LegalAddress, max_items=2)  # type: ignore
    contact_details: Optional[LegalContactDetails]
    remarks: NormalizedString = Field(default="", required=False, max_length=300)
    kyc_verification_details: KycDetails
    identity_details: conlist(LegalIdentity, min_items=1)  # type: ignore
    relation_person_details: Optional[List[LegalRelationPerson]]
